@@ -1,3 +1,9 @@
+/******************************************************************************
+* Copyright 1996-2013 United States Government as represented by the
+* Administrator of the National Aeronautics and Space Administration.
+* All Rights Reserved.
+******************************************************************************/
+
 import java.io.*;
 import java.text.*;
 import java.util.*;
@@ -66,6 +72,7 @@ import gsfc.nssdc.cdf.util.*;
       Oct 11, 2007     David Han        Added the -output:STDOUT option.
                                         Made "xsd" (schema) as the default
                                           from "dtd".
+      Oct 10, 2012     Mike Liu         Added handling for CDF_TIME_TT2000.
  *****************************************************************************/
 
 public class CDF2CDFML implements CDFConstants {
@@ -568,6 +575,16 @@ public class CDF2CDFML implements CDFConstants {
                     padValue = Epoch16.encode((double[]) padValue);
                   else
                     padValue = Epoch16.encode4((double[]) padValue);
+                } else if (dataType == CDF_TIME_TT2000) {
+                  padValue = CDFTT2000.encode(((Long) padValue).longValue());
+                } else if (dataType == CDF_CHAR || dataType == CDF_UCHAR) {
+		  char[] tmp = ((String)padValue).toCharArray();
+		  if (!((String)padValue).matches("[0-9A-Za-z !*#$%&'()*+,-./:;<=>?@\\\\\\[\\]^_`{|}~]*") || (tmp.length != (int) numElements)) {
+                    System.out.println ("Var: "+v.getName()+" padValue="+(String)padValue+ " invalid string... replaced by blank(s)...");
+                    char[] tmp2 = new char[(int)numElements];
+		    Arrays.fill(tmp2, ' ');
+		    padValue = new String(tmp2);
+		  }
                 }
                 out.print (" padValue=\""+padValue+"\"");
             }
@@ -908,6 +925,7 @@ public class CDF2CDFML implements CDFConstants {
             case (int)CDF_UCHAR:
             case (int)CDF_EPOCH:
 	    case (int)CDF_EPOCH16:
+	    case (int)CDF_TIME_TT2000:
                 out.println ("<string length=\""+MAX_STR+"\"/>");
                 break;
 
@@ -928,6 +946,10 @@ public class CDF2CDFML implements CDFConstants {
                 out.println ("<float width=\"18\" "+"precision=\"9\"/>");
                 break;
 
+            case (int)CDF_INT8:
+                out.println ("<integer width=\"20\"/>");
+                break;
+
             default:
                 break;
 
