@@ -1,3 +1,9 @@
+/******************************************************************************
+* Copyright 1996-2014 United States Government as represented by the
+* Administrator of the National Aeronautics and Space Administration.
+* All Rights Reserved.
+******************************************************************************/
+
 import java.awt.*;
 import java.awt.event.*;
 import java.io.*;
@@ -175,6 +181,10 @@ public class CDFConvert extends JFrame
     JTextField cdf2 = new JTextField("", 40);
 /** Text field for entering the optional skeleton table file name. */
     JTextField sklt = new JTextField("", 40);
+/** Text field for entering the optional sort variable. */
+    JTextField sort = new JTextField("", 5);
+/** Text field for entering the optional blockingFactor. */
+    JTextField blockingFactor = new JTextField("", 5);
 
 /** A label of "Source CDF:" for the text field cdf1. */
     JLabel CDF1 = new JLabel("Source CDF: ");
@@ -200,6 +210,10 @@ public class CDFConvert extends JFrame
     JLabel COMP = new JLabel("Compression: ");
 /** A label of "Checksum:" for the combobox majority. */
     JLabel CHECKSUM = new JLabel("Checksum:  ");
+/** A label of "Sort:" for sorting variable. */
+    JLabel SORT = new JLabel("Sort:  ");
+/** A label of "BlockingFactor:" for BlockingFactor. */
+    JLabel BlockingFactor = new JLabel("BlockingFactor:  ");
     
 /** Scroll pane for the text area. */
     JScrollPane scroller;
@@ -717,6 +731,14 @@ public class CDFConvert extends JFrame
         compression.addFocusListener(this);
         compression.addMouseListener(this);
         compression.setToolTipText("Enter the types of compression to be used with the destination CDF and/or variables");
+        sort.addActionListener(this);
+        sort.addFocusListener(this);
+        sort.addMouseListener(this);
+        sort.setToolTipText("Enter the variable name, .e.g., Epoch, that is used for sorting.");
+        blockingFactor.addActionListener(this);
+        blockingFactor.addFocusListener(this);
+        blockingFactor.addMouseListener(this);
+        blockingFactor.setToolTipText("Enter the blocking factor for all of the compressed variables.");
 	CASS.setFont(boldFont);
         cachesizes.setFont(boldFont);
         SPAR.setFont(boldFont);
