@@ -1,3 +1,9 @@
+/******************************************************************************
+* Copyright 1996-2014 United States Government as represented by the
+* Administrator of the National Aeronautics and Space Administration.
+* All Rights Reserved.
+******************************************************************************/
+
 import java.awt.*;
 import java.awt.event.*;
 import java.io.*;
@@ -42,8 +48,12 @@ public class CDFDump extends JFrame
     int abcX = 0;
 /** "Run it Batch" checkbox. */
     JCheckBox bac = new JCheckBox("Run it Batch");
-/** "Noheader" checkbox. */
-    JCheckBox qec = new JCheckBox("Noheader");
+/** "Header" checkbox. */
+    JCheckBox qec = new JCheckBox("Header",true);
+/** "Col2Row" checkbox. */
+    JCheckBox crc = new JCheckBox("Column-to-Row");
+/** "Detect" checkbox. */
+    JCheckBox dtc = new JCheckBox("Detect");
 
 /** A label of "Dump:" for the radio button zmode. */
     JLabel DUMP = new JLabel("Dump:  ");
@@ -53,9 +63,16 @@ public class CDFDump extends JFrame
     JRadioButton dump0 = new JRadioButton("All");
     JRadioButton dump1 = new JRadioButton("Data");
     JRadioButton dump2 = new JRadioButton("MetaData");
+    JRadioButton dump3 = new JRadioButton("Global");
+    JRadioButton dump4 = new JRadioButton("Variable");
 /** A radio button panel for Dump. */
     JPanel dumpPanel = new JPanel(true);
 
+/** A label of "Precision:" for the text field Precision. */
+    JLabel PRECISION = new JLabel("Significant #: ");
+/** Text field for entering optional variable names. */
+    JTextField precision = new JTextField("",2);
+
 /** A label of "Vars:" for the text field vars. */
     JLabel VARS = new JLabel("Variables: ");
 /** Text field for entering optional variable names. */
@@ -108,14 +125,17 @@ public class CDFDump extends JFrame
 /** The file separator. */
     String separator = System.getProperty("file.separator");
     String struf = "-format ";
+    String strcr = " ";
     String strqe = " ";
     String strdp = "all ";
+    String strdt = " ";
     String cdfspec = " ";
     String filespec = "";
     String xxx = " ";
     String cdfDelimiter = "";
     String strvars = "";
     String strrecs = "";
+    String strprec = "";
     String yyy;
     String zzz;
 
@@ -254,20 +274,31 @@ public class CDFDump extends JFrame
         dump0.setActionCommand("all");
         dump1.setActionCommand("data");
         dump2.setActionCommand("metadata");
+        dump3.setActionCommand("global");
+        dump4.setActionCommand("variable");
         dump0.addActionListener(dumpListener);
         dump1.addActionListener(dumpListener);
         dump2.addActionListener(dumpListener);
+        dump3.addActionListener(dumpListener);
+        dump4.addActionListener(dumpListener);
         dump0.setSelected(true);
         dump.add(dump0);
         dump.add(dump1);
         dump.add(dump2);
+        dump.add(dump3);
+        dump.add(dump4);
+        DUMP.setToolTipText("Select how to dump the CDF");
         dump0.setToolTipText("Dump all information from the CDF");
         dump1.setToolTipText("Dump only variable data from the CDF");
-        dump2.setToolTipText("Dump attributes from the CDF");
+        dump2.setToolTipText("Dump global and variable attributes from the CDF");
+        dump3.setToolTipText("Dump global attributes from the CDF");
+        dump4.setToolTipText("Dump variable attributes and entries from the CDF");
         dumpPanel.setLayout(new FlowLayout());
         dumpPanel.add(dump0);
         dumpPanel.add(dump1);
         dumpPanel.add(dump2);
+        dumpPanel.add(dump3);
+        dumpPanel.add(dump4);
 
         // Panel 1
 
@@ -308,39 +339,61 @@ public class CDFDump extends JFrame
 	makeJButton(panel2n, abc, 0, 1, 1, 1, 0.0, 0.0);
         makeJButton(panel2n, Box.createRigidArea(hpad40), 1, 1, 1, 1, 0.0, 0.0);
         makeJButton(panel2n, qec, 2, 1, 1, 1, 0.0, 0.0);
-//        makeJButton(panel2n, Box.createRigidArea(hpad40), 3, 1, 1, 1, 0.0, 0.0);
-	makeJButton(panel2n, ufc, 0, 2, 1, 1, 0.0, 0.0);
+        makeJButton(panel2n, Box.createRigidArea(hpad40), 3, 1, 1, 1, 0.0, 0.0);
+	makeJButton(panel2n, ufc, 4, 1, 1, 1, 0.0, 0.0);
+        makeJButton(panel2n, Box.createRigidArea(hpad40), 5, 1, 1, 1, 0.0, 0.0);
+	makeJButton(panel2n, crc, 6, 1, 1, 1, 0.0, 0.0);
+        makeJButton(panel2n, Box.createRigidArea(hpad40), 7, 1, 1, 1, 0.0, 0.0);
+
+        makeJButton(panel2n, dtc, 0, 2, 1, 1, 1.0, 1.0);
 	makeJButton(panel2n, Box.createRigidArea(hpad40), 1, 2, 1, 1, 0.0, 0.0);
-        makeJButton(panel2n, tpanel1, 2, 2, 1, 1, 0.0, 0.0);
+
         makeJButton(panel2n, Box.createRigidArea(vpad5), 0, 3, 1, 1, 0.0, 0.0);
+        makeJButton(panel2n, PRECISION, 2, 3, 1, 1, 1.0, 1.0);
+        makeJButton(panel2n, precision, 3, 3, 1, 1, 1.0, 1.0);
+	makeJButton(panel2n, Box.createRigidArea(hpad40), 4, 3, 1, 1, 0.0, 0.0);
+        makeJButton(panel2n, tpanel1, 6, 3, 1, 1, 0.0, 0.0);
+        makeJButton(panel2n, Box.createRigidArea(vpad5), 7, 3, 1, 1, 0.0, 0.0);
+
         makeJButton(panel2s, Box.createRigidArea(vpad5), 0, 0, 1, 1, 0.0, 0.0);
         makeJButton(panel2s, VARS, 0, 1, 1, 1, 1.0, 1.0);
         makeJButton(panel2s, vars, 1, 1, 2, 1, 1.0, 1.0);
         makeJButton(panel2s, RECS, 0, 2, 1, 1, 1.0, 1.0);
         makeJButton(panel2s, recs, 1, 2, 2, 1, 1.0, 1.0);
         makeJButton(panel2s, Box.createRigidArea(vpad20), 0, 3, 1, 1, 1.0, 1.0);
+
         panel2.add(panel2n, BorderLayout.NORTH);
         panel2.add(panel2s, BorderLayout.SOUTH);
 
         ufc.addItemListener(this);
 	ufc.setToolTipText("Select to use FORMAT attribute when displaying variable data");
+        crc.addItemListener(this);
+	crc.setToolTipText("Select to show column-major file in row-major form. Otherwise, show the data in their order in the file.");
 	abc.addItemListener(this);
         abc.setToolTipText("Display the current version");
         qec.addItemListener(this);
         qec.setToolTipText("Remove the 1-line header \"Dumping cdf from ...\" from the dump report");
         bac.addItemListener(this);
         bac.setToolTipText("Select to set up and run the dump by a batch job");
+        dtc.addItemListener(this);
+        dtc.setToolTipText("Select to detect and show only if the file is bad, otherwise no output");
+        precision.addActionListener(this);
+        precision.addMouseListener(this);
+        precision.addFocusListener(this);
+        PRECISION.setToolTipText("Enter the maximum significant number for showing floating-point values if FORMAT attribute entry is not available.");
         vars.addActionListener(this);
         vars.addMouseListener(this);
         vars.addFocusListener(this);
-        vars.setToolTipText("Enter the selected variable names, separated by comma, in the CDF to be dumped");
+        vars.setToolTipText("Enter the comma-separated variable names in the CDF to be dumped");
         recs.addActionListener(this);
         recs.addMouseListener(this);
         recs.addFocusListener(this);
         recs.setToolTipText("Enter the comma-separated record range for all variables in the CDF to be dumped");
 
+        VARS.setToolTipText("Enter the comma-separated variable names in the CDF to be dumped");
         VARS.setFont(boldFont);
         vars.setFont(boldFont);
+        RECS.setToolTipText("Enter the comma-separated record range for all variables in the CDF to be dumped");
         RECS.setFont(boldFont);
         recs.setFont(boldFont);
 
@@ -488,17 +541,36 @@ public class CDFDump extends JFrame
 		
 		if (ufc.isSelected())
 		  struf = "-format ";
-		else
+		else {
 		  struf = "-noformat ";
+                  precision.setText("");
+                }
+
+		if (dtc.isSelected())
+		  strdt = "-detect ";
+		else
+		  strdt = " ";
+
+		if (crc.isSelected())
+		  strcr = "-col2row ";
+		else {
+		  strcr = " ";
+                }
 
 		if (qec.isSelected())
-		  strqe = "-noheader ";
+		  strqe = "-header ";
 		else
-		  strqe = " ";
+		  strqe = "-noheader ";
 
-		commandX = command + struf + strqe + "-dump " + strdp + " " + 
-                           xxx + yyy + " " + zzz + " " + cdfDelimiter + cdfspec + 
-                           cdfDelimiter;
+                strprec = "";
+                if (struf.indexOf("noformat") == -1) {
+                  String precValue = precision.getText().trim();
+                  if (precValue.length() > 0) 
+                    strprec = new StringBuffer("-significant ").append(precValue).toString();
+                }
+		commandX = command + strdt + struf + strqe + "-dump " + strdp + " " + 
+                           xxx + yyy + " " + zzz + " " + strprec + " " + strcr +
+                           cdfDelimiter + cdfspec + cdfDelimiter;
 
                 String OS = System.getProperty("os.name").toLowerCase();
                 if (OS.indexOf("windows") != -1) {
@@ -600,8 +672,26 @@ public class CDFDump extends JFrame
         } else if (obj == ufc) { // "Use Format" checkbox
            if (ufc.isSelected())
                 struf = "-format ";
-           else
+           else {
                 struf = "-noformat ";
+                precision.setText("");
+           }
+        } else if (obj == dtc) { // "Use Format" checkbox
+           if (dtc.isSelected())
+                strdt = "-detect ";
+           else
+                strdt = " ";
+        } else if (obj == crc) { // "Column-to-Row" checkbox
+           if (crc.isSelected())
+                strcr = "-col2row ";
+           else {
+                strcr = " ";
+           }
+        } else if (obj == qec) { // "Header" checkbox
+           if (qec.isSelected())
+                strqe = "-header ";
+           else
+                strqe = "-noheader ";
 	} else if (obj == bac) { // "Run it Batch" checkbox
            if (bac.isSelected()) {
                 iba = 1;
