@@ -1,3 +1,9 @@
+/******************************************************************************
+* Copyright 1996-2014 United States Government as represented by the
+* Administrator of the National Aeronautics and Space Administration.
+* All Rights Reserved.
+******************************************************************************/
+
 import java.awt.*;
 import java.awt.event.*;
 import java.io.*;
@@ -372,16 +378,26 @@ public class CDFToolsDriver extends JFrame implements ActionListener {
         rbd.addActionListener(this);
         buttonGroup.add(rbd);
 
+        boolean jvmMatch = true;
+        int jvmMode = new Integer(System.getProperty("sun.arch.data.model")).intValue();
+        if (osName.toLowerCase().indexOf("windows") != -1) { /* check for JVM vs JNI */
+          if (checkJVM()) jvmMatch = true;
+          else jvmMatch = false;
+        }
         rb9 = new JRadioButton("CDFEdit");
+        rb9.getModel().setEnabled(jvmMatch);
 //      rb9.setMnemonic('9');
-        rb9.setToolTipText("Run the CDFEdit Tool Program");
+        if (jvmMatch) rb9.setToolTipText("Run the CDFEdit Tool Program");
+        else rb9.setToolTipText("Can't run this tool as your "+jvmMode+"-bit JVM does not match to "+(jvmMode==64?"32":"64")+"-bit CDF library");
         rb9.setActionCommand(cdfedit);
         rb9.addActionListener(this);
         buttonGroup.add(rb9);
 
         rba = new JRadioButton("CDFExport");
+        rba.getModel().setEnabled(jvmMatch);
 //      rba.setMnemonic('a');
-        rba.setToolTipText("Run the CDFExport Tool Program");
+        if (jvmMatch) rba.setToolTipText("Run the CDFExport Tool Program");
+        else rba.setToolTipText("Can't run this tool as your "+jvmMode+"-bit JVM does not match to "+(jvmMode==64?"32":"64")+"-bit CDF library");
         rba.setActionCommand(cdfexport);
         rba.addActionListener(this);
         buttonGroup.add(rba);
@@ -660,6 +676,32 @@ public class CDFToolsDriver extends JFrame implements ActionListener {
 
     }
 
+/** Check for JVM vs JNI for Windows.
+ */
+   private static boolean checkJVM() {
+
+	int jvm = new Integer(System.getProperty("sun.arch.data.model")).intValue();
+	String cdfBase = System.getenv("CDF_BASE");
+	String cmd = new String("\""+cdfBase+"\\bin\\unixfile.exe\" -m \""+cdfBase+
+			        "\\bin\\magic.mgc\" \""+cdfBase+"\\bin\\cdfdump.exe\"");
+	try {
+	  Runtime runtime = Runtime.getRuntime();
+	  Process process = runtime.exec(cmd);
+          BufferedReader inStream = null;
+          String line = null;
+          inStream = new BufferedReader(new
+                     InputStreamReader(process.getInputStream()), 8192);
+          line = inStream.readLine();
+          process.waitFor();
+	  int ix=line.indexOf("x86-64");
+	  if ((jvm == 64 && ix != -1) || (jvm == 32 && ix == -1)) return true;
+	  else return false;
+	} catch (Exception ex) {
+	  return true;
+	}
+
+   }
+
 /** The application to run the CDF Tool program driver. 
  */
     static public void main(String[] args) {
