@@ -1,3 +1,9 @@
+/******************************************************************************
+* Copyright 1996-2014 United States Government as represented by the
+* Administrator of the National Aeronautics and Space Administration.
+* All Rights Reserved.
+******************************************************************************/
+
 import java.awt.*;
 import java.awt.event.*;
 import java.io.*;
@@ -32,6 +38,15 @@ public class CDFValidate extends JFrame
 /** The panel for text area which goes to dialog's south portion. */
 //    JPanel panel4 = new JPanel(new BorderLayout(), true);
 
+/** A group radio buttons for DISPLAY */
+    ButtonGroup display = new ButtonGroup();
+/** A group radio buttons for zMode. */
+    JRadioButton debug = new JRadioButton("Debug");
+    JRadioButton quiet = new JRadioButton("Quiet");
+/** A radio button panel for display. */
+//    JPanel DISPLAY = new JPanel(new BorderLayout(), true);
+    JLabel DISPLAY = new JLabel("Display: ");
+
 /** "Debug" checkbox. */
     JCheckBox dbc = new JCheckBox("Debug",false);
 /** "Validate" checkbox. */
@@ -207,11 +222,33 @@ public class CDFValidate extends JFrame
         cdf.setFont(boldFont);
         dialog.getContentPane().add(panel1,BorderLayout.NORTH);
 
+        // Initialize the zMode choice.
+
+        DisplayListener displayListener = new DisplayListener();
+        debug.setActionCommand("1");
+        quiet.setActionCommand("2");
+        debug.addActionListener(displayListener);
+        quiet.addActionListener(displayListener);
+        display.add(debug);
+        display.add(quiet);
+        debug.setToolTipText("Displays the detailed validating processe");
+        quiet.setToolTipText("Shows nothing for a successful validation");
+
+        JPanel displayPanel = new JPanel(true);
+        JPanel tpanel1 = new JPanel(new BorderLayout(), true);
+        displayPanel.setLayout(new FlowLayout());
+        displayPanel.add(debug);
+        displayPanel.add(quiet);
+
+        tpanel1.add(DISPLAY,BorderLayout.WEST);
+        tpanel1.add(displayPanel, BorderLayout.CENTER);
+        tpanel1.add(Box.createRigidArea(hpad20), BorderLayout.EAST);
+
 	// Panel 2
 	makeJButton(panel2, Box.createRigidArea(hpad50), 0, 1, 1, 1, 1.0, 1.0);
 	makeJButton(panel2, vdc, 1, 1, 1, 1, 1.0, 1.0);
 	makeJButton(panel2, Box.createRigidArea(hpad50), 2, 1, 1, 1, 1.0, 1.0);
-	makeJButton(panel2, dbc, 3, 1, 1, 1, 1.0, 1.0);
+	makeJButton(panel2, tpanel1, 3, 1, 1, 1, 1.0, 1.0);
 	makeJButton(panel2, Box.createRigidArea(hpad50), 4, 1, 1, 1, 1.0, 1.0);
         makeJButton(panel2, abc, 5, 1, 1, 1, 1.0, 1.0);
 	makeJButton(panel2, Box.createRigidArea(vpad20), 0, 2, 1, 1, 1.0, 1.0);
@@ -315,6 +352,15 @@ public class CDFValidate extends JFrame
                   strvd = "-novalidate";
                 else
                   strvd = " ";
+                
+                if (!debug.isSelected() && !quiet.isSelected())
+                  strdb = "";
+                else if (debug.isSelected())
+                  strdb = "-debug ";
+                else if (quiet.isSelected())
+                  strdb = "-quiet ";
+                else 
+                  strdb = "";
 
                 commandX = command + strvd + " " + strdb + " " + 
                            cdfDelimiter + cdfspec + cdfDelimiter;
@@ -349,6 +395,15 @@ public class CDFValidate extends JFrame
 	}
     }
 
+/** Action for radio buttons for zMode
+ */
+
+    class DisplayListener implements ActionListener {
+        public void actionPerformed(ActionEvent e) {
+           String cmd = e.getActionCommand();
+        }
+    }
+
 /** Action for checkboxes selection
  */
     public void itemStateChanged(ItemEvent evt) {
@@ -372,11 +427,11 @@ public class CDFValidate extends JFrame
                 abc.setSelected(false);
 		abx = 0;
 	  }
-	} else if (obj == dbc) { // "Debug" checkbox
-		if (dbc.isSelected()) 
-			strdb = "-debug";
-		else
-			strdb = " ";
+//	} else if (obj == dbc) { // "Debug" checkbox
+//		if (dbc.isSelected()) 
+//			strdb = "-debug";
+//		else
+//			strdb = " ";
 	} else if (obj == vdc) { // "Validate" checkbox
 		if (!vdc.isSelected()) {
 			strvd = "-novalidate";
