@@ -1,4 +1,9 @@
-// $Id: EditAttrEntryDialog.java,v 1.1 2012/05/18 17:44:27 liu Exp $
+// $Id: EditAttrEntryDialog.java,v 1.3 2014/05/21 19:40:03 liu Exp $
+/******************************************************************************
+* Copyright 1996-2014 United States Government as represented by the
+* Administrator of the National Aeronautics and Space Administration.
+* All Rights Reserved.
+******************************************************************************/
 
 import javax.swing.*;
 import javax.swing.border.*;
@@ -398,16 +403,18 @@ public class EditAttrEntryDialog extends JDialog implements CDFConstants, Action
 
         cbdt.set(CDFUtils.getStringDataType(entry));
         tfne.set(""+numElements);
-        if (dataType == CDF_CHAR || dataType == CDF_UCHAR)
-          tfva.set("\"" + CDFUtils.getStringData(entry.getData(), ", ") +
-                         "\"");
-        else {
-	  int ii;
-	  if (dataType == CDF_EPOCH) ii = 1;
-	  else if (dataType == CDF_EPOCH16) ii = 2;
-	  else ii = 0;
-          tfva.set(CDFUtils.getStringData(entry.getData(), ", ", ii));
-	}
+        try {
+          if (dataType == CDF_CHAR || dataType == CDF_UCHAR)
+            tfva.set("\"" + CDFUtils.getStringData(entry.getData(), ", ") +
+                           "\"");
+          else {
+	    int ii;
+	    if (dataType == CDF_EPOCH) ii = 1;
+	    else if (dataType == CDF_EPOCH16) ii = 2;
+	    else ii = 0;
+            tfva.set(CDFUtils.getStringData(entry.getData(), ", ", ii));
+	  }
+        } catch (CDFException ex) {}
 	outData = (String) tfva.get();
         // Deprecated - tfva.requestDefaultFocus();
         tfva.requestFocus();
