@@ -1,3 +1,9 @@
+/******************************************************************************
+* Copyright 1996-2014 United States Government as represented by the
+* Administrator of the National Aeronautics and Space Administration.
+* All Rights Reserved.
+******************************************************************************/
+
 import java.awt.*;
 import java.awt.event.*;
 import java.text.*;
@@ -446,7 +452,9 @@ public class EditvVarEntryTableScrollPane extends JScrollPane implements CDFCons
                   dataObject[jj][2] = " ";
                 } else {
                   dataType = en.getDataType();
-                  value = en.getData();
+                  try {
+                    value = en.getData();
+                  } catch (CDFException ex) {}
 		  length = en.getNumElements();
                   dataObject[jj][0] = attr.toString();
                   dataObject[jj][1] = CDFUtils.getStringDataType(dataType) + "/" +
