@@ -1,3 +1,9 @@
+/******************************************************************************
+* Copyright 1996-2014 United States Government as represented by the
+* Administrator of the National Aeronautics and Space Administration.
+* All Rights Reserved.
+******************************************************************************/
+
 import java.awt.*;
 import java.awt.event.*;
 import java.io.*;
@@ -57,6 +63,9 @@ public class SkeletonCDF extends JFrame
 /** "Log Progress" checkbox. */
     JCheckBox lpc = new JCheckBox("Log Progress");
     int lpcX = 0;
+/** "Backward"  checkbox. */
+    JCheckBox bkc = new JCheckBox("Backward");
+    int bkcX = 0;
 /** "About" checkbox. */
     JCheckBox abc = new JCheckBox("About");
 /** "Run it Batch" checkbox. */
@@ -131,6 +140,7 @@ public class SkeletonCDF extends JFrame
     String strds = " ";
     String strde = " ";
     String strlp = " ";
+    String strbk = " ";
     String strzz = " ";
     String strzm = "";
     String report, Zmode, caches, temp;
@@ -360,7 +370,9 @@ public class SkeletonCDF extends JFrame
         makeJButton(panel2n, rec, 2, 2, 1, 1, 1.0, 1.0);
         makeJButton(panel2n, Box.createRigidArea(hpad40), 3, 2, 1, 1, 1.0, 1.0);
 	makeJButton(panel2n, tpanel1, 4, 2, 1, 1, 1.0, 1.0);
-        makeJButton(panel2n, abc, 0, 3, 1, 1, 1.0, 1.0);
+        makeJButton(panel2n, bkc, 0, 3, 1, 1, 1.0, 1.0);
+        makeJButton(panel2n, Box.createRigidArea(hpad40), 1, 3, 1, 1, 1.0, 1.0);
+        makeJButton(panel2n, abc, 2, 3, 1, 1, 1.0, 1.0);
 //        makeJButton(panel2n, bac, 2, 3, 1, 1, 1.0, 1.0);
 	makeJButton(panel2n, Box.createRigidArea(vpad10), 0, 4, 1, 1, 1.0, 1.0);
         makeJButton(panel2s, CASS, 0, 0, 1, 1, 1.0, 1.0);
@@ -381,6 +393,8 @@ public class SkeletonCDF extends JFrame
         rwc.setToolTipText("Select to report warning messages from the CDF library calls");
         lpc.addItemListener(this);
         lpc.setToolTipText("Select to display the progress");
+        bkc.addItemListener(this);
+        bkc.setToolTipText("Select to make bakward (V2.7) file");
         rec.addItemListener(this);
         rec.setToolTipText("Select to report error messages from the CDF library calls");
         dsc.addItemListener(this);
@@ -542,12 +556,12 @@ public class SkeletonCDF extends JFrame
                 else sktDelimiter = "\"";
 
                 commandX = command  + strzz + struf + strlp +
-			   strds + strde + report + Zmode + 
+			   strds + strde + strbk + report + Zmode + 
 			   Caches + cdfspec + sktDelimiter + skltspec +
 			   sktDelimiter;
                 /*************
 		commandX = command  + strzz + struf + strlp +
-			   strds + strde + report + Zmode + 
+			   strds + strde + strbk + report + Zmode + 
 			   Caches + cdfspec + " " + skltspec;
 		*************/
                 String OS = System.getProperty("os.name").toLowerCase();
@@ -691,6 +705,14 @@ public class SkeletonCDF extends JFrame
                   strlp = "-nolog ";
 		  lpcX = 0;
 		}
+        } else if (obj == bkc) { // "Backward" checkbox
+                if (bkc.isSelected()) {
+                  strbk = "-backward ";
+		  bkcX = 1;
+                } else {
+                  strbk = " ";
+		  bkcX = 0;
+		}
 	} else if (obj == zzc) { // "-0.0 to 0.0" checkbox
                 if (zzc.isSelected())
                   strzz = "-neg2posfp0 ";
