@@ -1,3 +1,9 @@
+/******************************************************************************
+* Copyright 1996-2013 United States Government as represented by the
+* Administrator of the National Aeronautics and Space Administration.
+* All Rights Reserved.
+******************************************************************************/
+
 import java.io.*;
 import java.text.*;
 import java.util.*;
@@ -126,9 +132,6 @@ public class ReadCDF implements CDFConstants{
                 for (Enumeration e1 =ent.elements() ; e1.hasMoreElements() ;) {
                     Entry entry = (Entry) e1.nextElement();
                     if (entry != null) {
-                       if (entry.getID() == 11) 
-                           ; // entry.delete();
-                       else {
                           long eDataType = entry.getDataType();
                           System.out.print ("\t"+entry.getID()+" ("+
                                          CDFUtils.getStringDataType(eDataType)+
@@ -140,10 +143,17 @@ public class ReadCDF implements CDFConstants{
                                 Epoch.encode(((Double) data).doubleValue()));
                           } else if (eDataType == CDF_EPOCH16) {
                               CDFUtils.printData (Epoch16.encode((double[])data));
-                          } else
-                              CDFUtils.printData (data);
+                          } else if (eDataType == CDF_TIME_TT2000) {
+                              CDFUtils.printData (
+                                CDFTT2000.encode(((Long)data).longValue()));
+                          } else {
+                            if (CDFUtils.isStringDataType(eDataType))
+                              System.out.print("\"");
+                            CDFUtils.printData (data);
+                            if (CDFUtils.isStringDataType(eDataType))
+                              System.out.print("\"");
+                          }
                           System.out.println (" ");
-                       }
                     }
                 }
                 System.out.println (" ");
@@ -188,8 +198,16 @@ public class ReadCDF implements CDFConstants{
                                Epoch.encode(((Double) data).doubleValue()));
                        } else if (eDataType == CDF_EPOCH16) {
                            CDFUtils.printData (Epoch16.encode((double[])data));
-                       } else
-                           CDFUtils.printData (data);
+                       } else if (eDataType == CDF_TIME_TT2000) {
+                           CDFUtils.printData (
+                               CDFTT2000.encode(((Long) data).longValue()));
+                       } else {
+                          if (CDFUtils.isStringDataType(eDataType))
+                            System.out.print("\"");
+                          CDFUtils.printData (data);
+                          if (CDFUtils.isStringDataType(eDataType))
+                            System.out.print("\"");
+                       }
                        System.out.println (" ");
                     }
                 }
@@ -208,7 +226,7 @@ public class ReadCDF implements CDFConstants{
               Vector  vars = cdf.getVariables();
 
               System.out.println("\nVariable Information ("+n+" variables)\n"+
-                                 "=========================================");
+                                 "===================================================");
               for (Enumeration e = vars.elements() ; e.hasMoreElements() ;) {
                 Variable v = (Variable) e.nextElement();
 
@@ -221,7 +239,7 @@ public class ReadCDF implements CDFConstants{
                 dt = v.getDataType();
                 dataType = CDFUtils.getStringDataType(dt);
                 dataType = dataType + "/" + String.valueOf(v.getNumElements());
-                noOfBlanks = 14 - dataType.length();
+                noOfBlanks = 20 - dataType.length();
                 for (i=0; i < noOfBlanks; i++)
                     dataType = dataType + " ";
 
@@ -231,7 +249,7 @@ public class ReadCDF implements CDFConstants{
                     if (i > 0) System.out.print (",");
                     System.out.print (dimSizes[i]); 
                 }
-                System.out.print ("]\t");
+                System.out.print ("]  ");
                 // if (numDims == 1) System.out.print ("\t");
 
                 System.out.print((v.getRecVariance() ? "T" : "F")+"/");
@@ -245,15 +263,16 @@ public class ReadCDF implements CDFConstants{
                      System.out.print (" sparseRecord=\""+sparseRecord+"\"");
 
                 // Print the user-define pad value if one exists. 
-		if (v.checkPadValueExistence()) {
+		// if (v.checkPadValueExistence()) {
                   padValue = v.getPadValue(); 
                   if (dt == CDF_EPOCH)
                     padValue = Epoch.encode(((Double) padValue).doubleValue());
                   else if (dt == CDF_EPOCH16)
                     padValue = Epoch16.encode((double[]) padValue);
-
-                  System.out.print (" PadValue=\""+padValue+"\"");
-                }
+                  else if (dt == CDF_TIME_TT2000)
+                    padValue = CDFTT2000.encode(((Long) padValue).longValue());
+                  System.out.print (" PadValue="+((CDFUtils.isStringDataType(dt)||CDFUtils.isEpochDataType(dt))?"\"":"")+padValue+((CDFUtils.isStringDataType(dt)||CDFUtils.isEpochDataType(dt))?"\"":""));
+                // }
 
                 blockingFactor = v.getBlockingFactor();
                 if (blockingFactor > 0)
@@ -309,7 +328,12 @@ public class ReadCDF implements CDFConstants{
 		    else if (myDT == CDF_EPOCH16)
 		      System.out.println ("Pad value:        "+
                          Epoch16.encode((double[])pv));
-                    else
+                    else if (myDT == CDF_TIME_TT2000)
+                      System.out.println ("Pad value:        "+
+                         CDFTT2000.toUTCstring(((Long)pv).longValue()));
+                    else if (CDFUtils.isStringDataType(myDT))
+                      System.out.println ("Pad value:        \""+ pv+"\"");
+                    else 
 		      System.out.println ("Pad value:        "+ pv);
 		  }
 		}
@@ -335,19 +359,17 @@ public class ReadCDF implements CDFConstants{
                 /********************************************************/
                 long status;
                 for (i=0; i <= maxRec; i++) {
-                     System.out.println ("Record # "+i+":");
                      data = v.getHyperDataObject(i, 1, 1,
                                                  dimIndices, 
                                                  dimSizes, 
                                                  dimIntervals);
                      status = cdf.getStatus();
-                     if (status != VIRTUAL_RECORD_DATA) {
-                         data.dumpData();
-                         System.out.println(" ");                
+                     if (status == CDF_OK || status != VIRTUAL_RECORD_DATA) {
+                       System.out.println ("Record # "+i+":");
+                       data.dumpData();
                      }
-                     else
-                         System.out.println("    <virtual>\n");
                 }
+                System.out.println("");
               }
 
               /**************************************************/
