@@ -1,5 +1,10 @@
-package gsfc.nssdc.cdf;
+/*
+ * Copyright 1996-2014 United States Government as represented by the
+ * Administrator of the National Aeronautics and Space Administration.
+ * All Rights Reserved.
+ */
 
+package gsfc.nssdc.cdf;
 /**
  * This class defines the constants used by the CDF library and CDF Java 
  * APIs, and it mimics the cdf.h include file from the cdf distribution.  
@@ -212,21 +217,22 @@ public interface CDFConstants {
     /* The default pad values for each data type */ 
     /*-------------------------------------------*/
 
-    public static final byte   DEFAULT_BYTE_PADVALUE	= (byte)0;
-    public static final byte   DEFAULT_INT1_PADVALUE	= (byte)0;
-    public static final short  DEFAULT_UINT1_PADVALUE	= (short)0;
-    public static final short  DEFAULT_INT2_PADVALUE	= (short)0;
-    public static final int    DEFAULT_UINT2_PADVALUE	= 0;
-    public static final int    DEFAULT_INT4_PADVALUE	= 0;
-    public static final long   DEFAULT_UINT4_PADVALUE	= 0L;
-    public static final long   DEFAULT_INT8_PADVALUE	= 0L;
-    public static final float  DEFAULT_REAL4_PADVALUE	= (float)0.0;
-    public static final float  DEFAULT_FLOAT_PADVALUE	= (float)0.0;
-    public static final double DEFAULT_REAL8_PADVALUE	= (double)0.0;
-    public static final double DEFAULT_DOUBLE_PADVALUE	= (double)0.0;
+    public static final byte   DEFAULT_BYTE_PADVALUE	= (byte)-127;
+    public static final byte   DEFAULT_INT1_PADVALUE	= (byte)-127;
+    public static final short  DEFAULT_UINT1_PADVALUE	= (short)254;
+    public static final short  DEFAULT_INT2_PADVALUE	= (short)-32767;
+    public static final int    DEFAULT_UINT2_PADVALUE	= 65534;
+    public static final int    DEFAULT_INT4_PADVALUE	= -2147483647;
+    public static final long   DEFAULT_UINT4_PADVALUE	= 4294967294L;
+    public static final long   DEFAULT_INT8_PADVALUE	= -9223372036854775807L;
+    public static final float  DEFAULT_REAL4_PADVALUE	= (float)-1.0E30;
+    public static final float  DEFAULT_FLOAT_PADVALUE	= (float)-1.0E30;
+    public static final double DEFAULT_REAL8_PADVALUE	= (double)-1.0E30;
+    public static final double DEFAULT_DOUBLE_PADVALUE	= (double)-1.0E30;
     public static final char   DEFAULT_CHAR_PADVALUE	= ' ';
     public static final char   DEFAULT_UCHAR_PADVALUE	= ' ';
     public static final double DEFAULT_EPOCH_PADVALUE	= (double)0.0;
+    public static final double DEFAULT_EPOCH16_PADVALUE	= (double)0.0;
     public static final long   DEFAULT_TT2000_PADVALUE	= -9223372036854775807L;
                                        /* == 1710-01-01T00.00.00.000000000 */
     /*-------------------*/
@@ -572,6 +578,8 @@ public interface CDFConstants {
     public static final long STAGE_CACHESIZE_ =	        154;
     public static final long COMPRESS_CACHESIZE_ =	155;
     public static final long CDF_CHECKSUM_ =            156;
+    public static final long rVAR_RECORDS_RENUMBER_ =   157;
+    public static final long zVAR_RECORDS_RENUMBER_ =   158;
 
     public static final long CDFwithSTATS_ = 200;   /* For CDF internal use only! */
     public static final long CDF_ACCESS_ =   201;   /* For CDF internal use only! */
