@@ -1,4 +1,10 @@
-//$Id: CDFData.java,v 1.1 2012/05/18 17:48:50 liu Exp $
+//$Id: CDFData.java,v 1.3 2014/11/10 17:23:36 liu Exp $
+/*
+ * Copyright 1996-2014 United States Government as represented by the
+ * Administrator of the National Aeronautics and Space Administration.
+ * All Rights Reserved.
+ */
+
 package gsfc.nssdc.cdf;
 
 import java.lang.*;
@@ -428,7 +434,6 @@ synchronized(v) {
         cmds.addElement(new Long(NULL_));
 
 	theData._myVar.getMyCDF().executeCommand((CDFObject)theData, cmds);
-
 	// unwrap the data
 	theData.arrayify();
 
@@ -983,7 +988,7 @@ synchronized(v) {
 	    if (len == 1) {
 //		if (dimCounts[0] > 1) 
               if (dimCounts[0] > 1 || (numDims > 0 && varDimSizes[0] == 1))
-		    nDims += 1;
+		    nDims ++;
 	    } else {
 		boolean multi=false;
 		for (int i=0; i<len; i++)
@@ -992,15 +997,15 @@ synchronized(v) {
 			nDims ++;
 	    }
 	    // Get the dimSizes
-	    dimSizes = new int[nDims];
-	    if (nDims == 1) {
+	    if (nDims > 0) {
+	      dimSizes = new int[nDims];
+	      if (nDims == 1) {
 		dimSizes[0] = ((recCount > 1) ? (int)recCount : (int)dimCounts[0]);
-	    } else {
+	      } else {
 		if (recCount > 1) {
 		    int j = 0;
 		    dimSizes[0] = (int)recCount;
 		    for (int i=0; i<numDims; i++) {
-//			if (dimCounts[i] > 1) {
                       if (dimCounts[i] > 1 || varDimSizes[i] == 1) {
 			   j++;
 			   dimSizes[j] = (int)dimCounts[i];
@@ -1009,18 +1014,19 @@ synchronized(v) {
 		} else {
 		    int j = -1;
 		    for (int i=0; i<numDims; i++) {
-//			if (dimCounts[i] > 1) {
                       if (dimCounts[i] > 1 || varDimSizes[i] == 1) {
 			   j++;
 			   dimSizes[j] = (int)dimCounts[i];
                       }
 		    }
 		}
+	      }
+              for (int dimN = 0; dimN < nDims; dimN++)
+                 numValues *= dimSizes[dimN];
+	    } else {
+              dimSizes = new int[1];
+	      dimSizes[0] = 1;
 	    }
-	    if (nDims > 0) {
-                for (int dimN = 0; dimN < nDims; dimN++)
-                     numValues *= dimSizes[dimN];
-            }
 	    numValues *= numElements;
 	}
     }
@@ -1098,11 +1104,17 @@ synchronized(v) {
 	    else {
 	      byte[] myBytes = ((String)padValue).getBytes();
 	      int ilen = myBytes.length;
-	      for (int i=0, j=0; i<numValues; i++, j++) {
-//		if (j == numElements) j = 0;
-                if (j == ilen) j = 0;
-		((byte [])_data)[i] = myBytes[j];
-	      }
+              if (ilen > 0) {
+	        for (int i=0, j=0; i<numValues; i++, j++) {
+//		  if (j == numElements) j = 0;
+                  if (j == ilen) j = 0;
+		  ((byte [])_data)[i] = myBytes[j];
+	        }
+              } else { /* nul-string... fill with spaces */
+	        for (int i=0; i<numValues; i++) {
+		  ((byte [])_data)[i] = ((byte[]) (new String(" ").getBytes()))[0];
+	        }
+              } 
 	    }
 	    break;
 	case (int)CDF_BYTE:
@@ -1264,8 +1276,8 @@ synchronized(v) {
             _data = new double [2*(int)numValues];
             if (padValue == null)
               for (int i=0; i<numValues; i++) {
-                 ((double [])_data)[2*i] = (double)DEFAULT_EPOCH_PADVALUE;
-                 ((double [])_data)[2*i+1] = (double)DEFAULT_EPOCH_PADVALUE;
+                 ((double [])_data)[2*i] = (double)DEFAULT_EPOCH16_PADVALUE;
+                 ((double [])_data)[2*i+1] = (double)DEFAULT_EPOCH16_PADVALUE;
               }
             else
                 for (int i=0; i<numValues; i++) {
