@@ -1,4 +1,9 @@
-//$Id: Entry.java,v 1.1 2012/05/18 17:48:50 liu Exp $
+//$Id: Entry.java,v 1.6 2014/05/22 14:11:50 liu Exp $
+/*
+ * Copyright 1996-2014 United States Government as represented by the
+ * Administrator of the National Aeronautics and Space Administration.
+ * All Rights Reserved.
+ */
 
 package gsfc.nssdc.cdf;
 
@@ -62,7 +67,6 @@ public class Entry extends Object implements CDFObject, CDFConstants {
      * The entry data
      */
     private Object data = null;
-    private Object xdata = null;
 
     /**
      * Since the data is a generic object and is set on-the-fly, this holds
@@ -158,6 +162,8 @@ synchronized(myAttribute) {
         /********************/
 	/* Get the data out */
         /********************/
+/* Not do it now... Perform it in getData method, instead. */
+/*
 	theEntry.buildDataObject();
 	cmds.removeAllElements();
 	item0.removeAllElements();
@@ -191,7 +197,7 @@ synchronized(myAttribute) {
 
 	theEntry.myAttribute.getMyCDF().
 	    executeCommand((CDFObject)theEntry, cmds);
-
+*/
 	myAttribute.addEntry(theEntry, (int)id);
 	return theEntry;
     }
@@ -498,7 +504,46 @@ synchronized(myAttribute) {
      *
      * @return the data for this entry 
      */
-    public synchronized Object getData() {
+    public synchronized Object getData() throws CDFException {
+        Vector cmds = new Vector();
+        Vector item0 = new Vector();
+        Vector item1 = new Vector();
+        Vector itemA = new Vector();
+        Vector itemB = new Vector();
+        Vector itemC = new Vector();
+
+        /********************/
+        /* Get the data out */
+        /********************/
+        this.buildDataObject();
+
+        cmds.addElement(new Long(SELECT_));
+          cmds.addElement(new Long(CDF_));
+            itemA.addElement("cdfID");
+            itemA.addElement("J");
+          cmds.addElement(itemA);
+          cmds.addElement(new Long(ATTR_));
+            itemB.addElement("attrID");
+            itemB.addElement("J");
+          cmds.addElement(itemB);
+          cmds.addElement(((this.scope == GLOBAL_SCOPE) ?
+                           new Long(gENTRY_) : new Long(zENTRY_)));
+            itemC.addElement("id");
+            itemC.addElement("J");
+          cmds.addElement(itemC);
+
+        cmds.addElement(new Long(GET_));
+          cmds.addElement(((this.scope == GLOBAL_SCOPE) ?
+                           new Long(gENTRY_DATA_) :
+                           new Long(zENTRY_DATA_)));
+            item0.addElement("data");
+            item0.addElement("Ljava/lang/Object;");
+          cmds.addElement(item0);
+        cmds.addElement(new Long(NULL_));
+
+        this.myAttribute.getMyCDF().
+            executeCommand((CDFObject)this, cmds);
+
 	return data;
     }
     
@@ -677,7 +722,6 @@ synchronized(myAttribute) {
           this.xdataType = this.dataType;
           this.xnumElements = this.numElements;
           this.xdataSignature = this.dataSignature;
-          this.xdata = this.data;
           this.dataType = dataType;
           this.numElements = CDFUtils.getNumElements(dataType, data);
           this.dataSignature = CDFUtils.getSignature(data);
@@ -691,7 +735,6 @@ synchronized(myAttribute) {
 	} catch (CDFException ex) {
 	  this.dataType = this.xdataType;
 	  this.numElements = this.xnumElements;
-	  this.data = this.xdata;
 	  this.dataSignature = this.xdataSignature;
 	  throw new CDFException(ex.getCurrentStatus());
 	}
