@@ -1,4 +1,10 @@
-// $Id: Variable.java,v 1.3 2012/06/27 20:49:17 liu Exp $
+// $Id: Variable.java,v 1.7 2014/11/10 17:23:36 liu Exp $
+/*
+ * Copyright 1996-2014 United States Government as represented by the
+ * Administrator of the National Aeronautics and Space Administration.
+ * All Rights Reserved.
+ */
+
 package gsfc.nssdc.cdf;
 
 import java.util.*;
@@ -344,7 +350,7 @@ synchronized(myCDF) {
      *                    <LI>CDF_UCHAR  - 1-byte, unsigned character
      *                    <LI>CDF_UINT1  - 1-byte, unsigned integer
      *                    <LI>CDF_INT2   - 2-byte, signed integer
-     *                    <LI>CDF_UNIT2  - 2-byte, unsigned integer
+     *                    <LI>CDF_UINT2  - 2-byte, unsigned integer
      *                    <LI>CDF_INT4   - 4-byte, signed integer
      *                    <LI>CDF_UINT4  - 4-byte, unsigned integer
      *                    <LI>CDF_INT8   - 8-byte, signed integer
@@ -986,7 +992,6 @@ synchronized(myCDF) {
                 interval[i] = 1L;
             }
         }
-
         return CDFData.get(this, recNum, numRecs, 1L, indices, sizes, interval);
     }
 
@@ -1756,7 +1761,7 @@ synchronized(myCDF) {
      * @return the ID of this variable 
      */
     public synchronized long getID() {
-	return myCDF.getVariableID(name);
+	  return myCDF.getVariableID(name);
     }
     
     /**
@@ -1934,7 +1939,9 @@ synchronized(myCDF) {
     }
 
     /**
-     * Deletes a range of records from this variable.
+     * Deletes a range of records from this variable. While non-sparse
+     * variable records after the last deleted record will be renumbered,
+     * the sparse variable records will not.
      *
      * @param firstRec the first record to be deleted
      * @param lastRec the last record to be deleted <P>
@@ -1943,6 +1950,27 @@ synchronized(myCDF) {
      */
     public synchronized void deleteRecords(long firstRec, long lastRec) 
 		throws CDFException {
+        this.toDeleteRecords(firstRec, lastRec, false);
+    }
+
+    /**
+     * Deletes a range of records from this variable. All records after the
+     * last deleted record are renumbered for both the sparse or non-sparse
+     * variable after the deletion. 
+     *
+     * @param firstRec the first record to be deleted
+     * @param lastRec the last record to be deleted <P>
+     *
+     * @exception CDFException if a problem occurs deleting records
+     */
+    public synchronized void deleteRecordsRenumber(long firstRec,
+                                                   long lastRec) 
+		throws CDFException {
+        this.toDeleteRecords(firstRec, lastRec, true);
+    }
+
+    private void toDeleteRecords(long firstRec, long lastRec, boolean reNumber) 
+		throws CDFException {
 
         Vector cmds = new Vector();
         Vector item0 = new Vector();
@@ -1961,7 +1989,8 @@ synchronized(myCDF) {
           cmds.addElement(itemB);
 
         cmds.addElement(new Long(DELETE_));
-           cmds.addElement(new Long(zVAR_RECORDS_));
+           cmds.addElement(new Long(
+                           reNumber?zVAR_RECORDS_RENUMBER_:zVAR_RECORDS_));
               item0.addElement("firstRec");
               item0.addElement("J");
            cmds.addElement(item0);
@@ -2235,6 +2264,18 @@ synchronized(myCDF) {
         Vector itemA = new Vector();
 	Vector itemB = new Vector();
 
+	if (this.dataType == CDF_CHAR || this.dataType == CDF_UCHAR) {
+	  int len = ((String)padValue).length();
+	  if  (len < this.numElements) { /* Fill space(s) */
+	    StringBuffer tmp = new StringBuffer((String)padValue);
+            int ix;
+            for (ix = 0; ix < (this.numElements - len); ++ix) tmp.append(' ');
+	    this.padValue = new String(tmp.toString());
+	  } else
+	    this.padValue = padValue;
+	} else
+	  this.padValue = padValue;
+
 	try {
           cmds.addElement(new Long(SELECT_));
             cmds.addElement(new Long(CDF_));
@@ -2253,35 +2294,31 @@ synchronized(myCDF) {
             cmds.addElement(item0);
           cmds.addElement(new Long(NULL_));
 
-          this.padValueX = this.padValue;
-          this.padValue = padValue;
+          this.padValueX = padValue;
           id = getID();
 	  cdfID = myCDF.getID();
 
           myCDF.executeCommand((CDFObject)this, cmds);
 
 	} catch (CDFException e) {
-	  this.padValue = padValueX;
 	  throw new CDFException(CANNOT_CHANGE);
 	}
     }
 
     /**
      * Checks if the pad value has been defined for this variable. 
-     * While the getPadValue() method always returns a pad value, it may
-     * simply be the default pad value (albeit the pad value was never
-     * defined by the user). 
      *
      * @return Whether the user-defined pad value exists. It is
      *         either true or false.  
      *         <UL>
      *             <LI>true - pad value has been specified.
      *             <LI>false - pad value is not specified.<P> 
-     *                    Note: The system default pad value is returned if 
-     *                    getPadValue() is called.
      *         </UL>
      * @exception CDFException if a problem occurs checking the existence of the
      *                         pad value
+     *
+     * Note: The getPadValue() method will return a pad value, whether it is
+     *       the user defined or the default.
      */
     public synchronized boolean checkPadValueExistence() throws CDFException {
 
@@ -2319,13 +2356,46 @@ synchronized(myCDF) {
     }
 
     /**
-     * Gets the pad value set for this variable.
+     * Gets the pad value for this variable.
      *
      * @return the pad value set for this variable <P>
+     * @exception CDFException if a problem occurs reading the pad value
      */
-    public synchronized Object getPadValue() {
+    public synchronized Object getPadValue() throws CDFException {
 
-	return padValue;
+        if (this.padValue != null) return this.padValue;
+        Vector cmds = new Vector();
+        Vector item0 = new Vector();
+        Vector itemA = new Vector();
+        Vector itemB = new Vector();
+
+        if (getDataType() == CDF_EPOCH16)
+          padValue = new double[2];
+
+        cmds.addElement(new Long(SELECT_));
+          cmds.addElement(new Long(CDF_));
+            itemA.addElement("cdfID");
+            itemA.addElement("J");
+          cmds.addElement(itemA);
+          cmds.addElement(new Long(zVAR_));
+            itemB.addElement("id");
+            itemB.addElement("J");
+          cmds.addElement(itemB);
+
+          cmds.addElement(new Long(GET_));
+            cmds.addElement(new Long(zVAR_PADVALUE_));
+              item0.addElement("padValue");
+              item0.addElement("Ljava/lang/Object;");
+            cmds.addElement(item0);
+          cmds.addElement(new Long(NULL_));
+
+          id = getID();
+          cdfID = myCDF.getID();
+
+        myCDF.executeCommand((CDFObject)this, cmds);
+        this.padValueX = padValue;
+
+        return padValue;
 
     }
 
@@ -2666,6 +2736,8 @@ synchronized(myCDF) {
      *         a pad value has been specified, then CDF_OK is returned. <P>
      * @exception CDFException if a problem occurs checking the 
      *                         existence of pad value. 
+     * Note: for NO_PADVALUE_SPECIFIED, getPadValue method will return the
+     *       default pad value.
      */
     public synchronized long confirmPadValue() throws CDFException {
 
