@@ -1,3 +1,9 @@
+/*
+ * Copyright 1996-2014 United States Government as represented by the
+ * Administrator of the National Aeronautics and Space Administration.
+ * All Rights Reserved.
+ */
+
 package gsfc.nssdc.cdf.util;
 
 import gsfc.nssdc.cdf.CDFException;
@@ -12,6 +18,13 @@ import java.text.*;
 import java.io.*;
 import java.lang.reflect.*;
 
+/**
+ * This class contains the handy utility routines (methods) called by the
+ * CDF applications to handle epoch data of CDF's CDF_TIME_TT2000 data type.
+ *
+ * @version 1.0
+ */
+
 public class CDFTT2000 implements gsfc.nssdc.cdf.CDFConstants {
 
     private static int JulianDateJ2000_12h =    2451545;
@@ -29,9 +42,9 @@ public class CDFTT2000 implements gsfc.nssdc.cdf.CDFConstants {
     private static long HOURinNanoSecs =        3600000000000L;
     private static long MINUTEinNanoSecs =      60000000000L;
     private static long T12hinNanoSecs =        43200000000000L;
-    /* Julian days for 1707-09-23 and 2292-04-09, the valid TT2000 range. */
-    private static long JDY17070923 =           2344794;
-    private static long JDY22920409 =           2558295;
+    /* Julian days for 1707-09-22 and 2292-04-11, the valid TT2000 range. */
+    private static long JDY17070922 =           2344793;
+    private static long JDY22920411 =           2558297;
 
 
 /* Number of Delta(dAT) expressions before leap seconds were introduced */
@@ -146,7 +159,7 @@ public class CDFTT2000 implements gsfc.nssdc.cdf.CDFConstants {
       if (yy <= 0 || mm < 0 || dd < 0) return 0;
       jday = JulianDay12h(yy, mm, dd);
       /* Y-M-D should be in the 1707-09-23 and 2292-04-9 range. */
-      if (jday < JDY17070923 || jday > JDY22920409) return 0;
+      if (jday < JDY17070922 || jday > JDY22920411) return 0;
       return 1;
     }
 
@@ -335,6 +348,20 @@ public class CDFTT2000 implements gsfc.nssdc.cdf.CDFConstants {
         }           
     }               
 
+    /**
+     *
+     * Find the environment variable "CDF_LEAPSECONDSTABLE" that is
+     * defined for the leap seconds table for CDF.
+     *
+     * @return the leap seconds environment variable if defined
+     *         null if not.
+     */
+
+    public static String getLeapSecondsTableEnvVar()
+    {
+          return System.getenv("CDF_LEAPSECONDSTABLE");
+    }
+
     private static void LoadLeapSecondsTable()
     {
       if (!tableChecked) {
@@ -343,7 +370,7 @@ public class CDFTT2000 implements gsfc.nssdc.cdf.CDFConstants {
         int ix, im, count = 0;
         tableChecked = true;
         try {
-          table = CDF.getLeapSecondsTableEnvVar();
+          table = CDFTT2000.getLeapSecondsTableEnvVar();
           if (table != null) {
             ArrayList[] myList = new ArrayList[6]; 
             for (ix = 0; ix < 6; ++ix)
@@ -399,8 +426,7 @@ public class CDFTT2000 implements gsfc.nssdc.cdf.CDFConstants {
             entryCnt = count;
             fromFile = 1;
           }
-        } catch (Exception xx) { System.out.println(">>>>"+xx);
-        } 
+        } catch (Exception xx) { } 
         if (fromFile == 0) {
           LTD = new double[NDAT][6];
           for (ix = 0; ix < NDAT; ++ix) {
@@ -431,14 +457,24 @@ public class CDFTT2000 implements gsfc.nssdc.cdf.CDFConstants {
          
     }
 
-    private static double LeapSecondsfromYMD (long iy, long im, long id)
+    /**
+     * 
+     * Find the leap seconds from a given, UTC-based year/month/day.
+     *
+     * @param year the year
+     * @param month the month
+     * @param day the day
+     * @return the leap second value (TAI-UTC)
+     */
+
+    public static double LeapSecondsfromYMD (long year, long month, long day)
     {
        int i, j;
        long m, n;
        double da;
        if (LTD == null) LoadLeapSecondsTable ();
        j = -1;
-       m = 12 * iy + im;
+       m = 12 * year + month;
        for (i = entryCnt-1; i >=0; --i) {
          n = (long) (12 * LTD[i][0] + LTD[i][1]);
          if (m >= n) {
@@ -451,7 +487,7 @@ public class CDFTT2000 implements gsfc.nssdc.cdf.CDFConstants {
        /* If pre-1972, adjust for drift. */
        if (j < NERA1) {
          double jda;
-         jda = JulianDay12h (iy, im, id);
+         jda = JulianDay12h (year, month, day);
          da += ((jda - MJDbase) - LTD[j][4]) * LTD[j][5];
        }
        return da;
@@ -482,7 +518,8 @@ public class CDFTT2000 implements gsfc.nssdc.cdf.CDFConstants {
 
     /**
      * 
-     * Breaks a TT2000 epoch value down into its full component parts.
+     * Breaks a TT2000 epoch value down into its full, UTC-based date/time
+     * component parts.
      *
      * @param nanoSecSinceJ2000 the epoch value, in nanoseconds since J2000, 
      *                          to break down
@@ -506,11 +543,7 @@ public class CDFTT2000 implements gsfc.nssdc.cdf.CDFConstants {
        if (nanoSecSinceJ2000 == FILLED_TT2000_VALUE)
          return new long[] {9999, 12, 31, 23, 59, 59, 999, 999, 999};
        if (nanoSecSinceJ2000 == DEFAULT_TT2000_PADVALUE) 
-         return new long[] {0, 1, 1, 0, 0, 0, 0, 0, 0, 0};
-/*
-       if (nanoSecSinceJ2000 == ILLEGAL_TT2000_VALUE) 
-         return new long[] {1710, 1, 1, 0, 0, 0, 0, 0, 0, 0};
-*/
+         return new long[] {0, 1, 1, 0, 0, 0, 0, 0, 0};
        double epoch, tmp1, offsecs;
        long secSinceJ2000, t2, t3;
        long nansec, milsec;
@@ -523,6 +556,7 @@ public class CDFTT2000 implements gsfc.nssdc.cdf.CDFConstants {
        toPlus = 0.0;
        t3 = nanoSecSinceJ2000;
        datx = LeapSecondsfromJ2000 (nanoSecSinceJ2000);
+/*
        nanoSecSinceJ2000 -= dTinNanoSecs;
        nanoSecSinceJ2000 += T12hinNanoSecs;
        t2 = nanoSecSinceJ2000;
@@ -532,6 +566,33 @@ public class CDFTT2000 implements gsfc.nssdc.cdf.CDFConstants {
          nansec = SECinNanoSecs + nansec;
          --secSinceJ2000;
        }
+*/
+/*
+       secSinceJ2000 = (long) ((double)nanoSecSinceJ2000/SECinNanoSecsD);
+       nansec = (long) (nanoSecSinceJ2000 - secSinceJ2000 * SECinNanoSecs);
+       secSinceJ2000 += 43200;
+       secSinceJ2000 -= 32;
+       nansec -= 184000000L;
+*/
+       if (nanoSecSinceJ2000 > 0) { /* try to avoid overflow (substraction first) */
+         secSinceJ2000 = (long) ((double)nanoSecSinceJ2000/SECinNanoSecsD);
+         nansec = (long) (nanoSecSinceJ2000 - secSinceJ2000 * SECinNanoSecs);
+         secSinceJ2000 -= 32; /* secs portion in dT */
+         secSinceJ2000 += 43200; /* secs in 12h */
+         nansec -= 184000000L; /* nanosecs portion in dT */
+       } else { /* try to avoid underflow (addition first) */
+         nanoSecSinceJ2000 += T12hinNanoSecs; /* 12h in nanosecs */
+         nanoSecSinceJ2000 -= dTinNanoSecs /* dT in nanosecs */;
+         secSinceJ2000 = (long) ((double)nanoSecSinceJ2000/SECinNanoSecsD);
+         nansec = (long) (nanoSecSinceJ2000 - secSinceJ2000 * SECinNanoSecs);
+       }
+
+       if (nansec < 0) {
+         nansec = SECinNanoSecs + nansec;
+         --secSinceJ2000;
+       }
+       t2 = secSinceJ2000 * SECinNanoSecs + nansec;
+
        if (datx[0] > 0.0) { /* Post-1972.... */
          secSinceJ2000 -= (long) datx[0];
          epoch = J2000Since0AD12hSec + secSinceJ2000;
@@ -643,7 +704,8 @@ public class CDFTT2000 implements gsfc.nssdc.cdf.CDFConstants {
 
     /**
      * 
-     * Breaks a TT2000 epoch value down into its full component parts.
+     * Breaks a TT2000 epoch value down into its full, UTC-based date/time
+     * component parts.
      *
      * @param nanoSecSinceJ2000 the epoch value, in nanoseconds since J2000, 
      *                          to break down
@@ -667,11 +729,49 @@ public class CDFTT2000 implements gsfc.nssdc.cdf.CDFConstants {
        return toUTCparts (nanoSecSinceJ2000);
     }
 
+    /**
+     * 
+     * Breaks an array of TT2000 epoch values down into two dimensional array,
+     * the first dimension being the count of the epoch values, and the second
+     * dimension being their full, UTC-based date/time component parts for
+     * each value.
+     *
+     * @param nanoSecSinceJ2000 an array of epoch values, in nanoseconds since
+     *                          J2000, to break down
+     * @return a 2-dim array of long, the first dimension being the value
+     *         count while the second dimension containing the epoch parts,
+     *         as follows:
+     *  <TABLE BORDER="0">
+     *    <TR><TD ALIGN="CENTER">Index</TD><TD ALIGN="CENTER">Part</TD></TR>
+     *    <TR><TD>0</TD><TD>year</TD></TR>
+     *    <TR><TD>1</TD><TD>month</TD></TR>
+     *    <TR><TD>2</TD><TD>day</TD></TR>
+     *    <TR><TD>3</TD><TD>hour</TD></TR>
+     *    <TR><TD>4</TD><TD>minute</TD></TR>
+     *    <TR><TD>5</TD><TD>second</TD></TR>
+     *    <TR><TD>6</TD><TD>millisecond</TD></TR>
+     *    <TR><TD>7</TD><TD>microsecond</TD></TR>
+     *    <TR><TD>8</TD><TD>nanosecond</TD></TR>
+     *  </TABLE>
+     *  or null if null or an empty input array is detected
+     */
+
+    public static long[][] breakdown (long[] nanoSecSinceJ2000) 
+    {
+       if (nanoSecSinceJ2000 == null) return null;
+       int len = nanoSecSinceJ2000.length;
+       if (len <= 0) return null;
+       long[][] comps = new long[len][9];
+       for (int i = 0; i < len; ++i) 
+         comps[i] = CDFTT2000.breakdown (nanoSecSinceJ2000[i]);
+       return comps;
+    }
+
     /** 
      * 
      * Computes a TT2000 epoch, nanoseconds since J2000, based on its 
-     * date/time component parts. The day field can contain a fraction
-     * of a day.
+     * UTC-based date/time component parts. The day field can contain a 
+     * fraction of a day.
      *
      * @param year the year, a full year in double
      * @param month the month, a full month in double
@@ -710,8 +810,8 @@ public class CDFTT2000 implements gsfc.nssdc.cdf.CDFConstants {
     /**
      * 
      * Computes a TT2000 epoch, nanoseconds since J2000, based on its 
-     * date/time component parts. The hour field can contain a fraction
-     * of a hour.
+     * UTC-based date/time component parts. The hour field can contain a
+     * fraction of a hour.
      *
      * @param year the year, a full year in double
      * @param month the month, a full month in double
@@ -751,7 +851,7 @@ public class CDFTT2000 implements gsfc.nssdc.cdf.CDFConstants {
     /**
      * 
      * Computes a TT2000 epoch, nanoseconds since J2000, based on its 
-     * date/time component parts. The minute field can contain a
+     * UTC-based date/time component parts. The minute field can contain a
      * fraction of a minute.
      *
      * @param year the year, a full year in double
@@ -792,7 +892,7 @@ public class CDFTT2000 implements gsfc.nssdc.cdf.CDFConstants {
     /**
      * 
      * Computes a TT2000 epoch, nanoseconds since J2000, based on its 
-     * date/time component parts. The second field can contain a
+     * UTC-based date/time component parts. The second field can contain a
      * fraction of a second.
      *
      * @param year the year, a full year in double
@@ -833,7 +933,7 @@ public class CDFTT2000 implements gsfc.nssdc.cdf.CDFConstants {
     /**
      * 
      * Computes a TT2000 epoch, nanoseconds since J2000, based on its 
-     * date/time component parts. The millisecond field can contain a
+     * UTC-based date/time component parts. The millisecond field can contain a
      * fraction of a millisecond.
      *
      * @param year the year, a full year in double
@@ -842,7 +942,7 @@ public class CDFTT2000 implements gsfc.nssdc.cdf.CDFConstants {
      * @param hour the hour, a full hour in double
      * @param minute the minute, a full minute in double
      * @param second the second, a full second in double
-     * @param msec the millisecond in double
+     * @param milsec the millisecond in double
      *
      * @return the TT2000 epoch value in long
      *
@@ -875,7 +975,7 @@ public class CDFTT2000 implements gsfc.nssdc.cdf.CDFConstants {
     /**
      * 
      * Computes a TT2000 epoch, nanoseconds since J2000, based on its 
-     * date/time component parts. The microsecond field can contain a
+     * UTC-based date/time component parts. The microsecond field can contain a
      * fraction, which usually indicates the end of the parameter list.
      *
      * @param year the year, a full year in double
@@ -884,8 +984,8 @@ public class CDFTT2000 implements gsfc.nssdc.cdf.CDFConstants {
      * @param hour the hour, a full hour in double
      * @param minute the minute, a full minute in double
      * @param second the second, a full second in double
-     * @param msec the millisecond, a full millisecond in double
-     * @param usec the microsecond in double
+     * @param milsec the millisecond, a full millisecond in double
+     * @param micsec the microsecond in double
      *
      * @return the TT2000 epoch value in long
      *
@@ -917,7 +1017,7 @@ public class CDFTT2000 implements gsfc.nssdc.cdf.CDFConstants {
     /**
      * 
      * Computes a TT2000 epoch, nanoseconds since J2000, based on its 
-     * date/time component parts. 
+     * UTC-based date/time component parts. 
      *
      * @param year the year, a full year in double
      * @param month the month, a full month in double
@@ -1095,13 +1195,8 @@ public class CDFTT2000 implements gsfc.nssdc.cdf.CDFConstants {
          return FILLED_TT2000_VALUE; 
        else if (lyl == 0 && lml == 1 && ldl == 1 && lhl == 0 && lnl == 0 &&
                 lsl == 0 && lll == 0 && lul == 0 && lal == 0)
-         return DEFAULT_TT2000_PADVALUE; 
-/*
-       else if (lyl == 1710 && lml == 1 && ldl == 1 && lhl == 0 && lnl == 0 &&
-                lsl == 0 && lll == 0 && lul == 0 && lal == 0)
-         return ILLEGAL_TT2000_VALUE; 
-*/
-       if ((year-lyl) != 0.0 || (month-lml) != 0.0) return ILLEGAL_TT2000_VALUE;
+         return DEFAULT_TT2000_PADVALUE;
+/*     if ((year-lyl) != 0.0 || (month-lml) != 0.0) return ILLEGAL_TT2000_VALUE; */
        if ((lyl<1708 || lyl > 2291) && (ValidateYMD(lyl,lml,ldl) == 0))
          return ILLEGAL_TT2000_VALUE;
        iy = 10000000 * lml + 10000 * ldl + lyl;
@@ -1114,16 +1209,24 @@ public class CDFTT2000 implements gsfc.nssdc.cdf.CDFConstants {
        jd = jd - JulianDateJ2000_12h;
        subDayinNanoSecs = lhl * HOURinNanoSecs + lnl * MINUTEinNanoSecs +
                           lsl * SECinNanoSecs + lll * 1000000 + lul * 1000 + lal;
-       nanoSecSinceJ2000 = (long) (jd * DAYinNanoSecs) + subDayinNanoSecs;
+       nanoSecSinceJ2000 = (long) jd * DAYinNanoSecs + subDayinNanoSecs;
        t2 = (long) (currentLeapSeconds * SECinNanoSecs);
-       nanoSecSinceJ2000 = nanoSecSinceJ2000 - T12hinNanoSecs + t2 + dTinNanoSecs;
+       if (nanoSecSinceJ2000 < 0) {
+         nanoSecSinceJ2000 += t2;
+         nanoSecSinceJ2000 += dTinNanoSecs;
+         nanoSecSinceJ2000 -= T12hinNanoSecs;
+       } else {
+         nanoSecSinceJ2000 -= T12hinNanoSecs;
+         nanoSecSinceJ2000 += t2;
+         nanoSecSinceJ2000 += dTinNanoSecs;
+       }
        return nanoSecSinceJ2000;
     }
 
     /**
      * 
      * Computes a TT2000 epoch, nanoseconds since J2000, based on its 
-     * date/time component parts. 
+     * UTC-based date/time component parts. 
      *
      * @param year the year, a full year in long
      * @param month the month, a full month in long
@@ -1153,7 +1256,259 @@ public class CDFTT2000 implements gsfc.nssdc.cdf.CDFConstants {
 
     /**
      * 
-     * Convert an epoch in TT2000 value to CDF_EPOCH value.
+     * Computes an array of TT2000 epochs, nanoseconds since J2000, based on
+     * its UTC-based date/time component parts. 
+     *
+     * @param year an array of years, a full year in long
+     * @param month an array of months, a full month in long
+     * @param day an array of days, a full day in long
+     * 
+     * @return the TT2000 epoch values in long or null if an invalid input
+     *         array(s) is detected
+     * 
+     * @exception CDFException an TT2000_TIME_ERROR if an illegal 
+     *                         component value is detected.
+     */
+
+    public static long[] compute (long[] year, long[] month, long[] day)
+           throws CDFException
+    {
+       if (year == null || month == null || day == null) return null;
+       int len = year.length;
+       if (len != month.length || len != day.length) return null;
+       long[] tt2000s = new long[len];
+       for (int i = 0; i < len; ++i)
+         tt2000s[i] = CDFTT2000.compute (year[i], month[i], day[i], 0L,
+                                         0L, 0L, 0L, 0L, 0L);
+       return tt2000s;
+    }
+
+    /**
+     * 
+     * Computes an array of TT2000 epochs, nanoseconds since J2000, based on
+     * its UTC-based date/time component parts. 
+     *
+     * @param year an array of years, a full year in long
+     * @param month an array of months, a full month in long
+     * @param day an array of days, a full day in long
+     * @param hour an array of hours, a full hour in long
+     * 
+     * @return the TT2000 epoch value in long or null if an invalid input
+     *         array(s) is detected
+     * 
+     * @exception CDFException an TT2000_TIME_ERROR if an illegal 
+     *                         component value is detected.
+     */
+
+    public static long[] compute (long[] year, long[] month, long[] day,
+                                  long[] hour)
+           throws CDFException
+    {
+       if (year == null || month == null || day == null ||
+           hour == null) return null;
+       int len = year.length;
+       if (len != month.length || len != day.length || len != hour.length)
+         return null;
+       long[] tt2000s = new long[len];
+       for (int i = 0; i < len; ++i)
+         tt2000s[i] = CDFTT2000.compute (year[i], month[i], day[i], hour[i],
+                                         0L, 0L, 0L, 0L, 0L);
+       return tt2000s;
+    }
+
+    /**
+     * 
+     * Computes an array of TT2000 epochs, nanoseconds since J2000, based on
+     * its * UTC-based date/time component parts. 
+     *
+     * @param year an array of years, a full year in long
+     * @param month an array of months, a full month in long
+     * @param day an array of days, a full day in long
+     * @param hour an array of hours, a full hour in long
+     * @param minute an array of minutes, a full minute in long
+     * 
+     * @return the TT2000 epoch values in long or null if an invalid input
+     *         array(s) is detected
+     * 
+     * @exception CDFException an TT2000_TIME_ERROR if an illegal 
+     *                         component value is detected.
+     */
+
+    public static long[] compute (long[] year, long[] month, long[] day,
+                                  long[] hour, long[] minute)
+           throws CDFException
+    {
+       if (year == null || month == null || day == null ||
+           hour == null || minute == null) return null;
+       int len = year.length;
+       if (len != month.length || len != day.length || len != hour.length ||
+           len != minute.length) return null;
+       long[] tt2000s = new long[len];
+       for (int i = 0; i < len; ++i)
+         tt2000s[i] = CDFTT2000.compute (year[i], month[i], day[i], hour[i],
+                                         minute[i], 0L, 0L, 0L, 0L);
+       return tt2000s;
+    }
+
+    /**
+     * 
+     * Computes an array of TT2000 epochs, nanoseconds since J2000, based on
+     * its * UTC-based date/time component parts. 
+     *
+     * @param year an array of years, a full year in long
+     * @param month an array of months, a full month in long
+     * @param day an array of days, a full day in long
+     * @param hour an array of hours, a full hour in long
+     * @param minute an array of minutes, a full minute in long
+     * @param second an array of seconds, a full second in long
+     * 
+     * @return the TT2000 epoch value in long or null if an invalid input
+     *         array(s) is detected
+     * 
+     * @exception CDFException an TT2000_TIME_ERROR if an illegal 
+     *                         component value is detected.
+     */
+
+    public static long[] compute (long[] year, long[] month, long[] day,
+                                  long[] hour, long[] minute, long[] second)
+           throws CDFException
+    {
+       if (year == null || month == null || day == null ||
+           hour == null || minute == null || second == null) return null;
+       int len = year.length;
+       if (len != month.length || len != day.length || len != hour.length ||
+           len != minute.length || len != second.length) return null;
+       long[] tt2000s = new long[len];
+       for (int i = 0; i < len; ++i)
+         tt2000s[i] = CDFTT2000.compute (year[i], month[i], day[i], hour[i],
+                                         minute[i], second[i], 0L, 0L, 0L);
+       return tt2000s;
+    }
+
+    /**
+     * 
+     * Computes an array of TT2000 epochs, nanoseconds since J2000, based on
+     * its * UTC-based date/time component parts. 
+     *
+     * @param year an array of years, a full year in long
+     * @param month an array of months, a full month in long
+     * @param day an array of days, a full day in long
+     * @param hour an array of hours, a full hour in long
+     * @param minute an array of minutes, a full minute in long
+     * @param second an array of seconds, a full second in long
+     * @param msec an array of milliseconds, a full millisecond in long
+     * 
+     * @return the TT2000 epoch values in long or null if an invalid input
+     *         array(s) is detected
+     * 
+     * @exception CDFException an TT2000_TIME_ERROR if an illegal 
+     *                         component value is detected.
+     */
+
+    public static long[] compute (long[] year, long[] month, long[] day,
+                                  long[] hour, long[] minute, long[] second,
+                                  long[] msec)
+           throws CDFException
+    {
+       if (year == null || month == null || day == null ||
+           hour == null || minute == null || second == null ||
+           msec == null) return null;
+       int len = year.length;
+       if (len != month.length || len != day.length || len != hour.length ||
+           len != minute.length || len != second.length || len != msec.length)
+         return null;
+       long[] tt2000s = new long[len];
+       for (int i = 0; i < len; ++i)
+         tt2000s[i] = CDFTT2000.compute (year[i], month[i], day[i], hour[i],
+                                         minute[i], second[i], msec[i], 0L, 0L);
+       return tt2000s;
+    }
+
+    /**
+     * 
+     * Computes an array of TT2000 epochs, nanoseconds since J2000, based on
+     * its * UTC-based date/time component parts. 
+     *
+     * @param year an array of years, a full year in long
+     * @param month an array of months, a full month in long
+     * @param day an array of days, a full day in long
+     * @param hour an array of hours, a full hour in long
+     * @param minute an array of minutes, a full minute in long
+     * @param second an array of seconds, a full second in long
+     * @param msec an array of milliseconds, a full millisecond in long
+     * @param usec an array of microseconds, a full microsecond in long
+     * 
+     * @return the TT2000 epoch value in long or null if an invalid input
+     *          array(s) is detected
+     * 
+     * @exception CDFException an TT2000_TIME_ERROR if an illegal 
+     *                         component value is detected.
+     */
+
+    public static long[] compute (long[] year, long[] month, long[] day,
+                                  long[] hour, long[] minute, long[] second,
+                                  long[] msec, long[] usec)
+           throws CDFException
+    {
+       if (year == null || month == null || day == null ||
+           hour == null || minute == null || second == null ||
+           msec == null || usec == null) return null;
+       int len = year.length;
+       if (len != month.length || len != day.length || len != hour.length ||
+           len != minute.length || len != second.length || len != msec.length ||
+           len != usec.length) return null;
+       long[] tt2000s = new long[len];
+       for (int i = 0; i < len; ++i)
+         tt2000s[i] = CDFTT2000.compute (year[i], month[i], day[i], hour[i],
+                                         minute[i], second[i], msec[i],
+                                         usec[i], 0L);
+       return tt2000s;
+    }
+
+    /**
+     * 
+     * Computes an array of TT2000 epochs, nanoseconds since J2000, based on
+     * its * UTC-based date/time component parts. 
+     *
+     * @param year an array of years, a full year in long
+     * @param month an array of months, a full month in long
+     * @param day an array of days, a full day in long
+     * @param hour an array of hours, a full hour in long
+     * @param minute an array of minutes, a full minute in long
+     * @param second an array of seconds, a full second in long
+     * @param msec an array of milliseconds, a full millisecond in long
+     * @param usec an array of microseconds, a full microsecond in long
+     * @param nsec an array of nanoseconds, a full nanosecond in long
+     * 
+     * @return the TT2000 epoch values in long or null if an invalid input
+     *         array(s) is detected
+     * 
+     * @exception CDFException an TT2000_TIME_ERROR if an illegal 
+     *                         component value is detected.
+     */
+
+    public static long[] compute (long[] year, long[] month, long[] day,
+                                  long[] hour, long[] minute, long[] second,
+                                  long[] msec, long[] usec, long[] nsec)
+           throws CDFException
+    {
+       if (year == null || month == null || day == null ||
+           hour == null || minute == null || second == null ||
+           msec == null || usec == null || nsec == null) return null;
+       int len = year.length;
+       if (len != month.length || len != day.length || len != hour.length ||
+           len != minute.length || len != second.length || len != msec.length ||           len != usec.length || len != nsec.length) return null;
+       long[] tt2000s = new long[len];
+       for (int i = 0; i < len; ++i)
+         tt2000s[i] = CDFTT2000.compute (year[i], month[i], day[i], hour[i],
+                                         minute[i], second[i], msec[i],
+                                         usec[i], nsec[i]);
+       return tt2000s;
+    }
+
+    /**
+     * 
+     * Convert an epoch value in TT2000 type to CDF_EPOCH type.
      *
      * @param nanoSecSinceJ2000 the nanoseconds since J2000
      *
@@ -1183,7 +1538,7 @@ public class CDFTT2000 implements gsfc.nssdc.cdf.CDFConstants {
     }
 
     /**
-     * Convert an epoch value in CDF_EPOCH to TT2000.
+     * Convert an epoch value in CDF_EPOCH type to TT2000 type.
      * Reset the predefined CDF_EPOCH value of -1.0E31, -1.0E-31, or the
      * default 0.0 or -0.0 (all are invalid for TT2000) to 0. 
      *
@@ -1212,7 +1567,7 @@ public class CDFTT2000 implements gsfc.nssdc.cdf.CDFConstants {
     }
 
     /**
-     * Convert an epoch in TT2000 value to CDF_EPOCH16 value.
+     * Convert an epoch value in TT2000 type to CDF_EPOCH16 type.
      *
      * @param nanoSecSinceJ2000 the nanoseconds since J2000
      * @param epoch the returned CDF_EPOCH16 value, a double[2] object
@@ -1252,7 +1607,7 @@ public class CDFTT2000 implements gsfc.nssdc.cdf.CDFConstants {
     }
 
     /**
-     * Convert an epoch data in CDF_EPOCH16 to TT2000.
+     * Convert an epoch data in CDF_EPOCH16 type to TT2000 type.
      * Reset the predefined CDF_EPOCH value of -1.0E31, -1.0E-31, or the
      * default 0.0 or -0.0 (all are invalid for TT2000) to 0. 
      *
@@ -1286,8 +1641,8 @@ public class CDFTT2000 implements gsfc.nssdc.cdf.CDFConstants {
 
     /**
      * 
-     * Converts an epoch value in TT2000 form into a readable date/time string
-     * of ISO 8601 formats.
+     * Converts an epoch value in TT2000 type into a readable, UTC-based 
+     * date/time string of ISO 8601 formats.
      *
      * <PRE>
      *
@@ -1308,8 +1663,8 @@ public class CDFTT2000 implements gsfc.nssdc.cdf.CDFConstants {
  
     /**
      * 
-     * Converts an epoch value in TT2000 form into a readable date/time string
-     * of ISO 8601 formats.
+     * Converts an epoch value in TT2000 type into a readable, UTC-based
+     * date/time string of ISO 8601 formats.
      *
      * <PRE>
      *            yyyy-mm-ddThh:mm:ss.ccccccccc
@@ -1331,8 +1686,8 @@ public class CDFTT2000 implements gsfc.nssdc.cdf.CDFConstants {
  
     /**
      * 
-     * Converts an epoch value in TT2000 form into a readable date/time string
-     * of chosen formats.
+     * Converts an epoch value in TT2000 type into a readable, UTC-based
+     * date/time string of chosen formats.
      *
      * <PRE>
      * Format:0   dd-mmm-yyyy hh:mm:ss.ccccccccc
@@ -1368,8 +1723,8 @@ public class CDFTT2000 implements gsfc.nssdc.cdf.CDFConstants {
  
     /**
      * 
-     * Converts an epoch value in TT2000 form into a readable date/time string
-     * of chosen formats.
+     * Converts an epoch value in TT2000 type into a readable, UTC-based 
+     * date/time string of chosen formats.
      *
      * <PRE>
      * Format:0   dd-mmm-yyyy hh:mm:ss.ccccccccc
@@ -1412,14 +1767,6 @@ public class CDFTT2000 implements gsfc.nssdc.cdf.CDFConstants {
          if (format == 2) return new String("00000101000000");
          if (format == 3) return new String("0000-01-01T00:00:00.000000000");
        }
-/*
-       if (nanoSecSinceJ2000 == ILLEGAL_TT2000_VALUE) {
-         if (format == 0) return new String("01-JAN-1710 00:00:00.000000000");
-         if (format == 1) return new String("17100101.0000000000");
-         if (format == 2) return new String("17100101000000");
-         if (format == 3) return new String("1710-01-01T00:00:00.000000000");
-       }
-*/
        long ly, lm, ld, lh, ln, ls, ll, lu, la;
        if (format < 0 || format > 3) format = 3;
        long[] datetime = toUTCparts(nanoSecSinceJ2000);
@@ -1496,8 +1843,8 @@ public class CDFTT2000 implements gsfc.nssdc.cdf.CDFConstants {
 
     /**
      * 
-     * Converts an epoch value in TT2000 form into a readable date/time string
-     * in ISO 8601 format.
+     * Converts an epoch value in TT2000 type into a readable, UTC-based 
+     * date/time string in ISO 8601 format.
      *
      * <PRE>
      *            yyyy-mm-ddThh:mm:ss.ccccccccc
@@ -1516,8 +1863,36 @@ public class CDFTT2000 implements gsfc.nssdc.cdf.CDFConstants {
  
     /**
      * 
-     * Converts an epoch value in TT2000 form into a readable date/time string
-     * of chosen formats.
+     * Converts an array of epoch values in TT2000 type into an array of
+     * readable, UTC-based date/time strings in ISO 8601 format.
+     *
+     * <PRE>
+     *            yyyy-mm-ddThh:mm:ss.ccccccccc
+     * Examples:  1990-04-01T03:05:02.000000000
+     *            1993-10-10T23:45:49.777888999
+     * </PRE>
+     *
+     * @param nanoSecSinceJ2000 the TT2000 epoch value, nanoseconds since
+     *                          J2000, in long
+     *
+     * @return strings representing the epochs or null if null or an empty
+     *         input array is detected
+     */
+    public static String[] encode (long[] nanoSecSinceJ2000)
+    {
+       if (nanoSecSinceJ2000 == null) return null;
+       int len = nanoSecSinceJ2000.length;
+       if (len <= 0) return null;
+       String[] tt2000 = new String[len];
+       for (int i = 0; i < len; ++i)
+         tt2000[i] = CDFTT2000.encode (nanoSecSinceJ2000[i]);
+       return tt2000;
+    }
+ 
+    /**
+     * 
+     * Converts an epoch value in TT2000 type into a readable, UTC-based 
+     * date/time string of chosen formats.
      *
      * <PRE>
      * Format:0   dd-mmm-yyyy hh:mm:ss.ccccccccc
@@ -1553,10 +1928,10 @@ public class CDFTT2000 implements gsfc.nssdc.cdf.CDFConstants {
 
     /**
      * 
-     * This method parses an input date/time string and returns a TT2000
-     * epoch value, nanoseconds since J2000.  The string must be in one of the 
-     * formats as shown below. Month abbreviations may be in any case and are
-     * always the first three letters of the month.
+     * This method parses an input, UTC-based, date/time string and returns a
+     * TT2000 epoch value, nanoseconds since J2000.  The string must be in
+     * one of the formats as shown below. Month abbreviations may be in any
+     * case and are always the first three letters of the month.
      *
      * <PRE>
      * Format:0   dd-mmm-yyyy hh:mm:ss.ccccccccc
@@ -1611,11 +1986,6 @@ public class CDFTT2000 implements gsfc.nssdc.cdf.CDFConstants {
            if (ly == 0 && lm == 1 && ld == 1 && lh == 0 &&
                ln == 0 && ls == 0 && tmp == 0)
              return DEFAULT_TT2000_PADVALUE;
-/*
-           if (ly == 1710 && lm == 1 && ld == 1 && lh == 0 &&
-               ln == 0 && ls == 0 && tmp == 0)
-             return ILLEGAL_TT2000_VALUE;
-*/
            if (tmp == 0) {
              ll = lu = la = 0;
            } else {
@@ -1643,10 +2013,6 @@ public class CDFTT2000 implements gsfc.nssdc.cdf.CDFConstants {
              return FILLED_TT2000_VALUE;
            if (ly == 0 && lm == 1 && ld == 1 && tmp == 0) 
              return DEFAULT_TT2000_PADVALUE;
-/*
-           if (ly == 1710 && lm == 1 && ld == 1 && tmp == 0) 
-             return ILLEGAL_TT2000_VALUE;
-*/
            if (tmp != 0) {
              int digits = new Long(tmp).toString().length();
              fraction = ld + (double) tmp / Math.pow(10.0, digits+1);
@@ -1668,10 +2034,6 @@ public class CDFTT2000 implements gsfc.nssdc.cdf.CDFConstants {
                ln == 59 && ls == 59)  return FILLED_TT2000_VALUE;
            if (ly == 0 && lm == 1 && ld == 1 && lh == 0 &&
                ln == 0 && ls == 0)  return DEFAULT_TT2000_PADVALUE;
-/*
-           if (ly == 1710 && lm == 1 && ld == 1 && lh == 0 &&
-               ln == 0 && ls == 0)  return ILLEGAL_TT2000_VALUE;
-*/
            return fromUTCparts ((double) ly, (double) lm, (double) ld,
                                 (double) lh, (double) ln, (double) ls,
                                 0.0, 0.0, 0.0);
@@ -1696,10 +2058,6 @@ public class CDFTT2000 implements gsfc.nssdc.cdf.CDFConstants {
            if (ly == 0 && lm == 1 && ld == 1 && lh == 0 &&
                ln == 0 && ls == 0 && tmp == 0)
              return DEFAULT_TT2000_PADVALUE;
-/*
-           if (ly == 1710 && lm == 1 && ld == 1 && lh == 0 &&
-               ln == 0 && ls == 0 && tmp == 0) return ILLEGAL_TT2000_VALUE;
-*/
            if (tmp == 0) {
              ll = lu = la = 0;
            } else {
@@ -1721,9 +2079,49 @@ public class CDFTT2000 implements gsfc.nssdc.cdf.CDFConstants {
     }
 
     /**
-     * 
+     *
      * This method parses an input date/time string and returns a TT2000
-     * epoch value, nanoseconds since J2000.  The string must be in one of the 
+     * epoch value, nanoseconds since J2000.  The string must be in one of the
+     * formats as shown below. Month abbreviations may be in any case and are
+     * always the first three letters of the month.
+     *
+     * <PRE>
+     * Format:0   dd-mmm-yyyy hh:mm:ss.ccccccccc
+     * Examples:  01-Apr-1990 03:05:02.000000000
+     *            10-Oct-1993 23:45:49.777888999
+     *
+     * Format:1   yyyymmdd.cccccccccc
+     * Examples:  19900401.1234567890
+     *            19931010.9998887776
+     *
+     * Format:2   yyyymmddhhmmss
+     * Examples:  19900401030502
+     *            19931010234549
+     *
+     * Format:3   yyyy-mm-ddThh:mm:ss.ccccccccc
+     * Examples:  1990-04-01T03:05:02.000000000
+     *            1993-10-10T23:45:49.777888999
+     * </PRE>
+     *
+     * These formats are the same as those created by toUTCstring.
+     *
+     * @param string the epoch in string representation
+     *
+     * @return A TT2000 epoch the epoch value in nanoseconds since J2000
+     *
+     * @exception CDFException an TT2000_TIME_ERROR if an illegal
+     *                         component value is detected.
+     */
+    public static long parse (String string) throws CDFException
+    {
+       return fromUTCstring (string);
+    }
+
+    /**
+     * 
+     * This method parses an array of input date/time strings and returns an
+     * array of  TT2000 epoch values, nanoseconds since J2000.
+     * The string must be in one of the 
      * formats as shown below. Month abbreviations may be in any case and are
      * always the first three letters of the month.
      *
@@ -1747,21 +2145,29 @@ public class CDFTT2000 implements gsfc.nssdc.cdf.CDFConstants {
      *
      * These formats are the same as those created by toUTCstring.
      *
-     * @param string the epoch in string representation
+     * @param strings an array of epochs in string representation
      *
-     * @return A TT2000 epoch the epoch value in nanoseconds since J2000
+     * @return TT2000 epoch values in nanoseconds since J2000 or null if null
+     *         or an empty input array is detected
      *
      * @exception CDFException an TT2000_TIME_ERROR if an illegal 
      *                         component value is detected.
      */
-    public static long parse (String string) throws CDFException
+    public static long[] parse (String[] strings) throws CDFException
     {
-       return fromUTCstring (string);
+       if (strings == null) return null;
+       int len = strings.length;
+       if (len <= 0) return null;
+       long[] tt2000 = new long[len];
+       for (int i = 0; i < len; ++i)
+         tt2000[i] = CDFTT2000.parse (strings[i]);
+       return tt2000;
     }
 
+    /**
     /**
      * 
-     * This method returns the last date that a leap second was added in
+     * This method returns the last UTC date that a leap second was added in
      * the leap second table used in the class. This can be used to check
      * whether the table is up-to-date.
      *
@@ -1779,8 +2185,8 @@ public class CDFTT2000 implements gsfc.nssdc.cdf.CDFConstants {
 
     /**
      * 
-     * This method converts the date/time in a GregorianCalendar class object
-     * to TT2000 time.
+     * This method converts the UTC-based date/time in a GregorianCalendar
+     * class object to TT2000 time.
      *
      * @return the TT2000 time
      * @exception CDFException an TT2000_TIME_ERROR if a GregorianCalendar 
@@ -1812,15 +2218,29 @@ public class CDFTT2000 implements gsfc.nssdc.cdf.CDFConstants {
 
     /**
      * 
-     * This method converts the date/time in TT2000 to a GregorianCalendar
-     * class object.
+     * This method converts the UTC-based date/time in TT2000 type to a
+     * GregorianCalendar class object in default, local time zone.
      *
      * @return a GregorianCalendar object
      */
     public static GregorianCalendar toGregorianTime (long tt2000)
+    {
+       return CDFTT2000.toGregorianTime (tt2000,
+                                         new GregorianCalendar().getTimeZone());
+    }
+
+    /**
+     * 
+     * This method converts the UTC-based date/time in TT2000 type to a
+     * GregorianCalendar class object in specified time zone.
+     *
+     * @return a GregorianCalendar object
+     */
+    public static GregorianCalendar toGregorianTime (long tt2000, TimeZone tz)
     {
        long[] date = CDFTT2000.toUTCparts(tt2000);
        GregorianCalendar gc = new GregorianCalendar();
+       gc.setTimeZone(tz);
        gc.set((int)date[0], (int)date[1]-1, (int)date[2], (int)date[3],
               (int)date[4], (int)date[5]);
        gc.set(Calendar.MILLISECOND, (int)date[6]);
