@@ -1,4 +1,9 @@
-// $Id: CDFUtils.java,v 1.1 2012/05/18 17:47:39 liu Exp $
+// $Id: CDFUtils.java,v 1.2 2014/05/14 17:38:34 liu Exp $
+/*
+ * Copyright 1996-2014 United States Government as represented by the
+ * Administrator of the National Aeronautics and Space Administration.
+ * All Rights Reserved.
+ */
 
 package gsfc.nssdc.cdf.util;
 
@@ -146,7 +151,7 @@ public class CDFUtils implements CDFConstants {
      */
     public static void printData (Object data) {
 
-	printData(data, 0);
+	printData(data, 0, false);
     }
 
     /**
@@ -169,46 +174,54 @@ public class CDFUtils implements CDFConstants {
             arrayLength = Array.getLength(data);
             for (i=0; i < arrayLength; i=i+jj) {
                  if (signature.charAt(1) == 'B')
-                     System.out.print (Array.getByte(data,i)+" ");
+                     System.out.print (Array.getByte(data,i)+
+                                       (i==(arrayLength-jj)?"":" "));
 
                  else if (signature.charAt(1) == 'S')
-                     System.out.print (Array.getShort(data,i)+" ");
+                     System.out.print (Array.getShort(data,i)+
+                                       (i==(arrayLength-jj)?"":" "));
 
                  else if (signature.charAt(1) == 'I')
-                     System.out.print (Array.getInt(data,i)+" ");
+                     System.out.print (Array.getInt(data,i)+
+                                       (i==(arrayLength-jj)?"":" "));
 
                  else if (signature.charAt(1) == 'J')
                      if (which == 0)
-                       System.out.print (Array.getLong(data,i)+" ");
+                       System.out.print (Array.getLong(data,i)+
+                                         (i==(arrayLength-jj)?"":" "));
                      else
-                       System.out.print (CDFTT2000.encode(Array.getLong(data,i))+" ");
+                       System.out.print (CDFTT2000.encode(Array.getLong(data,i))+
+                                         (i==(arrayLength-jj)?"":" "));
 
                  else if (signature.charAt(1) == 'F')
-                     System.out.print (Array.getFloat(data,i)+" ");
+                     System.out.print (Array.getFloat(data,i)+
+                                       (i==(arrayLength-jj)?"":" "));
 
                  else if (signature.charAt(1) == 'D')
 		   if (which == 1) {
-		     System.out.print (((!iso8601)?
-                                        Epoch.encode(Array.getDouble(data,i)):
-                                        Epoch.encode4(Array.getDouble(data,i)))+
-                                       " ");
+		     System.out.print ((!iso8601)?
+                                       Epoch.encode(Array.getDouble(data,i))+
+                                        (i==(arrayLength-jj)?"":" "):
+                                       Epoch.encode4(Array.getDouble(data,i))+
+                                        (i==(arrayLength-jj)?"":" "));
                    } else if (which == 2) {
                      double[] mmm = new double[2];
                      mmm[0] = Array.getDouble(data,i);
                      mmm[1] = Array.getDouble(data,i+1);
-                     System.out.print (((!iso8601)?
-                                        Epoch16.encode(mmm):
-                                        Epoch16.encode4(mmm))+
+                     System.out.print ((!iso8601)?
+                                       Epoch16.encode(mmm):
+                                       Epoch16.encode4(mmm)+
                                         (arrayLength==2?"":" "));
 		   } else
-                     System.out.print (Array.getDouble(data,i)+" ");
+                     System.out.print (Array.getDouble(data,i)+
+                                       (i==(arrayLength-jj)?"":" "));
             }
         }
         else {
 	   if (which == 1) 
-             System.out.print (((!iso8601)?
-                                Epoch.encode(((Double)data).doubleValue()):
-                                Epoch.encode4(((Double)data).doubleValue())));
+             System.out.print ((!iso8601)?
+                               Epoch.encode(((Double)data).doubleValue()):
+                               Epoch.encode4(((Double)data).doubleValue()));
            else if (which == 3)
              System.out.print (CDFTT2000.encode(((Long)data).longValue()));
            else
@@ -528,6 +541,31 @@ public class CDFUtils implements CDFConstants {
     }
 
 
+    /**
+     * Returns whether a CDF data type is a string type.
+     *
+     * @param dataType the data type to be examined <P>
+     * @return true if it is (CDF_CHAR or CDF_UCHAR)
+     *         false otherwise
+     */
+    public static boolean isStringDataType(long dataType) {
+        return (dataType == CDF_CHAR || dataType == CDF_UCHAR);
+    }
+
+
+    /**
+     * Returns whether a CDF data type is an epoch related type.
+     *
+     * @param dataType the data type to be examined <P>
+     * @return true if it is (CDF_EPOCH, CDF_EPOCH16 or CDF_TIME_TT2000)
+     *         false otherwise
+     */
+    public static boolean isEpochDataType(long dataType) {
+        return (dataType == CDF_EPOCH || dataType == CDF_EPOCH16 ||
+                dataType == CDF_TIME_TT2000);
+    }
+
+
     /**
      *  Gets the string representation of the given CDF data type. <P>
      *
