@@ -1,4 +1,10 @@
-// $Id: Epoch.java,v 1.1 2012/05/18 17:47:39 liu Exp $
+// $Id: Epoch.java,v 1.4 2014/08/20 17:39:16 liu Exp $
+/*
+ * Copyright 1996-2014 United States Government as represented by the
+ * Administrator of the National Aeronautics and Space Administration.
+ * All Rights Reserved.
+ */
+
 package gsfc.nssdc.cdf.util;
 
 import gsfc.nssdc.cdf.CDFException;
@@ -9,6 +15,10 @@ import java.util.*;
 import java.text.*;
 
 /**
+ * This class contains the handy utility routines (methods) called by the
+ * CDF applications to handle epoch data of CDF's CDF_EPOCH data type.
+ *
+ * @version 1.0
  *
  * <B>Example:</B>
  * <PRE> 
@@ -54,7 +64,7 @@ public class Epoch implements gsfc.nssdc.cdf.CDFConstants {
     /**
      * 
      * This function parses an input date/time string and returns an EPOCH
-     * value.  The format must be exactly as shown below.  
+     * value.  The string format must be exactly as shown below.  
      * Month abbreviations may be in any case and are always the first 
      * three letters of the month.
      *
@@ -64,14 +74,14 @@ public class Epoch implements gsfc.nssdc.cdf.CDFConstants {
      *           10-Oct-1993 23:45:49.999
      * </PRE>
      *
-     * The expected format is the same as that produced by encodeEPOCH.
+     * The expected format is the same as that produced by encode.
      *
      * @param inString the epoch in string representation
      * @return the value of the epoch represented by inString
      *
      * @exception CDFException if a bad epoch value is passed in inString
      */
-    public static double parse(String inString)
+    public static double parse (String inString)
 	throws CDFException
     {
 	long 
@@ -97,7 +107,7 @@ public class Epoch implements gsfc.nssdc.cdf.CDFConstants {
 	    monthStr = st.nextToken();
 	    year = Long.parseLong(st.nextToken());
 	    for (int monthX = 1; monthX <= 12; monthX++) {
-		if (monthStr.equals(_monthToken[monthX-1])) {
+		if (monthStr.equalsIgnoreCase(_monthToken[monthX-1])) {
 		    month = monthX;
 		    break;
 		}
@@ -120,6 +130,39 @@ public class Epoch implements gsfc.nssdc.cdf.CDFConstants {
 	}
     }
 
+    /**
+     * 
+     * This function parses an array of date/time strings and returns an array 
+     * of EPOCH values.  The string format must be exactly as shown below.  
+     * Month abbreviations may be in any case and are always the first 
+     * three letters of the month.
+     *
+     * <PRE>
+     * Format:   dd-mmm-yyyy hh:mm:ss.mmm
+     * Examples:  1-Apr-1990 03:05:02.000
+     *           10-Oct-1993 23:45:49.999
+     * </PRE>
+     *
+     * The expected format is the same as that produced by encode.
+     *
+     * @param inStrings the epochs in string representation
+     *
+     * @return the values of the epoch represented by inStrings or null if null
+     *         or an empty input array is detected
+     *
+     * @exception CDFException if a bad epoch value is passed in inStrings
+     */
+    public static double[] parse (String[] inStrings)
+	throws CDFException
+    {
+	if (inStrings == null) return null;
+	int len = inStrings.length;
+	double[] epochs = new double[len];
+	for (int i = 0; i < len; ++i)
+	  epochs[i] = parse (inStrings[i]);
+	return epochs;
+    }
+
     /**
      * 
      * This function parses an input date/time string and returns an EPOCH
@@ -133,7 +176,7 @@ public class Epoch implements gsfc.nssdc.cdf.CDFConstants {
      *            19671231.58      (== 19671213.5800000)
      * </PRE>
      *
-     * The expected format is the same as that produced by encodeEPOCH1.
+     * The expected format is the same as that produced by encode1.
      *
      * @param inString the epoch in string representation
      * @return the value of the epoch represented by inString
@@ -144,13 +187,13 @@ public class Epoch implements gsfc.nssdc.cdf.CDFConstants {
 	throws CDFException
     {
 	StringTokenizer st = new StringTokenizer(inString, ".");
-	String date = st.nextToken();
-	String time = st.nextToken();
-	long year = 0, month = 0, day = 0, hour = 0, minute = 0, 
-	    second = 0, msec = 0, fractionL = 0;
-	double fraction = 0.0;
+        try {
+	  String date = st.nextToken();
+	  String time = st.nextToken();
+	  long year = 0, month = 0, day = 0, hour = 0, minute = 0, 
+	       second = 0, msec = 0, fractionL = 0;
+	  double fraction = 0.0;
 
-	try {
 	    year   = Long.parseLong(date.substring(0,4));
 	    month  = Long.parseLong(date.substring(4,6));
 	    day    = Long.parseLong(date.substring(6));
@@ -165,7 +208,7 @@ public class Epoch implements gsfc.nssdc.cdf.CDFConstants {
 	    fraction -= (double) (second / 86400.0);
 	    msec = (long) (fraction * 86400000.0);
 	    return compute(year, month, day, hour, minute, second, msec);
-	} catch (java.lang.NumberFormatException e) {
+	} catch (Exception e) {
 	    throw new CDFException(ILLEGAL_EPOCH_FIELD);
 	}
     }
@@ -181,7 +224,7 @@ public class Epoch implements gsfc.nssdc.cdf.CDFConstants {
      *           19671231235959
      * </PRE>
      *
-     * The expected format is the same as that produced by encodeEPOCH2.
+     * The expected format is the same as that produced by encode2.
      *
      * @param inString the epoch in string representation
      * @return the value of the epoch represented by inString
@@ -200,7 +243,7 @@ public class Epoch implements gsfc.nssdc.cdf.CDFConstants {
 	    minute = Long.parseLong(inString.substring(10,12));
 	    second = Long.parseLong(inString.substring(12));
 	    return compute(year, month, day, hour, minute, second, 0L);
-	} catch (java.lang.NumberFormatException e) {
+	} catch (Exception e) {
 	    throw new CDFException(ILLEGAL_EPOCH_FIELD);
 	}
     }
@@ -216,7 +259,7 @@ public class Epoch implements gsfc.nssdc.cdf.CDFConstants {
      *            1993-10-10T23:45:49.999Z
      * </PRE>
      *
-     * The expected format is the same as that produced by encodeEPOCH3.
+     * The expected format is the same as that produced by encode3.
      *
      * @param inString the epoch in string representation
      * @return the value of the epoch represented by inString
@@ -270,7 +313,7 @@ public class Epoch implements gsfc.nssdc.cdf.CDFConstants {
      *            1993-10-10T23:45:49.999
      * </PRE>
      *
-     * The expected format is the same as that produced by encodeEPOCH3.
+     * The expected format is the same as that produced by encode3.
      *
      * @param inString the epoch in string representation
      * @return the value of the epoch represented by inString
@@ -339,6 +382,35 @@ public class Epoch implements gsfc.nssdc.cdf.CDFConstants {
 			"<dom.02>-<month>-<year> <hour>:<min>:<sec>.<fos>");
     }
 
+    /**
+     * 
+     * Converts an array of EPOCH values into an array of  readable date/time
+     * strings.
+     *
+     * <PRE>
+     * Format:    dd-mmm-yyyy hh:mm:ss.ccc
+     * Examples:  01-Apr-1990 03:05:02.000
+     *            10-Oct-1993 23:45:49.999
+     * </PRE>
+     *
+     * This format is the same as that expected by parse.
+     *
+     * @param epochs the epoch values
+     *
+     * @return Strings representing the epochs or null if null or an empty
+     *         input array is detected
+     */
+    public static String[] encode(double[] epochs)
+    {
+	if (epochs == null) return null;
+	int len = epochs.length;
+        if (len <= 0) return null;
+	String[] strings = new String[len];
+	for (int i = 0; i < len; ++i)
+          strings[i] = Epoch.encode (epochs[i]);
+	return strings;
+    }
+
     /**
      * 
      * Converts an EPOCH value into a readable date/time string.
@@ -791,6 +863,187 @@ public class Epoch implements gsfc.nssdc.cdf.CDFConstants {
 	return (86400000.0 * daysSince0AD + (double) msecInDay);
     }
 
+    /**
+     * 
+     * Computes an array of EPOCH values based on their component parts.
+     *
+     * @param year the year array
+     * @param month the month array
+     * @param day the day array
+     *
+     * @return an array of epoch values or null if an invalid input array(s)
+     *         is detected.
+     *
+     * @exception CDFException an ILLEGAL_EPOCH_FIELD if an illegal 
+     *    component value is detected.
+     */
+    public static double[] compute(long[] year, 
+				   long[] month, 
+				   long[] day) 
+	throws CDFException
+    {
+	if (year == null || month == null || day == null) return null;
+	int len = year.length;
+	if (len <= 0) return null;
+        if (len != month.length || len != day.length) return null;
+	double[] epochs = new double[len];
+	for (int i = 0; i < len; ++i)
+	  epochs[i] = Epoch.compute (year[i], month[i], day[i], 0L,
+				     0L, 0L, 0L);
+	return epochs;
+    }
+
+    /**
+     * 
+     * Computes an array of EPOCH values based on their component parts.
+     *
+     * @param year the year array
+     * @param month the month array
+     * @param day the day array
+     * @param hour the hour array
+     *
+     * @return an array of epoch values  or null if an invalid input array(s)
+     *         is detected
+     *
+     * @exception CDFException an ILLEGAL_EPOCH_FIELD if an illegal 
+     *    component value is detected.
+     */
+    public static double[] compute(long[] year, 
+				   long[] month, 
+				   long[] day, 
+				   long[] hour)
+	throws CDFException
+    {
+	if (year == null || month == null || day == null ||
+	    hour == null) return null;
+	int len = year.length;
+	if (len <= 0) return null;
+        if (len != month.length || len != day.length ||
+	    len != hour.length) return null;
+	double[] epochs = new double[len];
+	for (int i = 0; i < len; ++i)
+	  epochs[i] = Epoch.compute (year[i], month[i], day[i], hour[i],
+				     0L, 0L, 0L);
+	return epochs;
+    }
+
+    /**
+     * 
+     * Computes an array of EPOCH values based on their component parts.
+     *
+     * @param year the year array
+     * @param month the month array
+     * @param day the day array
+     * @param hour the hour array
+     * @param minute the minute array
+     *
+     * @return an array of epoch values  or null if an invalid input array(s)
+     *         is detected
+     *
+     * @exception CDFException an ILLEGAL_EPOCH_FIELD if an illegal 
+     *    component value is detected.
+     */
+    public static double[] compute(long[] year, 
+				   long[] month, 
+				   long[] day, 
+				   long[] hour,
+				   long[] minute)
+	throws CDFException
+    {
+	if (year == null || month == null || day == null ||
+	    hour == null || minute == null) return null;
+	int len = year.length;
+	if (len <= 0) return null;
+        if (len != month.length || len != day.length ||
+	    len != hour.length || len != minute.length) return null;
+	double[] epochs = new double[len];
+	for (int i = 0; i < len; ++i)
+	  epochs[i] = Epoch.compute (year[i], month[i], day[i], hour[i],
+				     minute[i], 0L, 0L);
+	return epochs;
+    }
+
+    /**
+     * 
+     * Computes an array of EPOCH values based on their component parts.
+     *
+     * @param year the year array
+     * @param month the month array
+     * @param day the day array
+     * @param hour the hour array
+     * @param minute the minute array
+     * @param second the second  array
+     *
+     * @return an array of epoch values  or null if an invalid input array(s)
+     *         is detected
+     *
+     * @exception CDFException an ILLEGAL_EPOCH_FIELD if an illegal 
+     *    component value is detected.
+     */
+    public static double[] compute(long[] year, 
+				   long[] month, 
+				   long[] day, 
+				   long[] hour,
+				   long[] minute, 
+				   long[] second)
+	throws CDFException
+    {
+	if (year == null || month == null || day == null ||
+	    hour == null || minute == null || second == null) return null;
+	int len = year.length;
+	if (len <= 0) return null;
+        if (len != month.length || len != day.length ||
+	    len != hour.length || len != minute.length ||
+	    len != second.length) return null;
+	double[] epochs = new double[len];
+	for (int i = 0; i < len; ++i)
+	  epochs[i] = Epoch.compute (year[i], month[i], day[i], hour[i],
+				     minute[i], second[i], 0L);
+	return epochs;
+    }
+
+    /**
+     * 
+     * Computes an array of EPOCH values based on their component parts.
+     *
+     * @param year the year array
+     * @param month the month array
+     * @param day the day array
+     * @param hour the hour array
+     * @param minute the minute array
+     * @param second the second array
+     * @param msec the millisecond array
+     *
+     * @return an array of epoch values  or null if an invalid input array(s)
+     *         is detected
+     *
+     * @exception CDFException an ILLEGAL_EPOCH_FIELD if an illegal 
+     *    component value is detected.
+     */
+    public static double[] compute(long[] year, 
+				   long[] month, 
+				   long[] day, 
+				   long[] hour,
+				   long[] minute, 
+				   long[] second,
+				   long[] msec)
+	throws CDFException
+    {
+	if (year == null || month == null || day == null ||
+	    hour == null || minute == null || second == null ||
+	    msec == null) return null;
+	int len = year.length;
+	if (len <= 0) return null;
+        if (len != month.length || len != day.length ||
+	    len != hour.length || len != minute.length ||
+	    len != second.length || len != msec.length) return null;
+	double[] epochs = new double[len];
+	for (int i = 0; i < len; ++i)
+	  epochs[i] = Epoch.compute (year[i], month[i], day[i], hour[i],
+				     minute[i], second[i], msec[i]);
+	return epochs;
+    }
+
     /**
      * 
      * Breaks an EPOCH value down into its component parts.
@@ -857,7 +1110,37 @@ public class Epoch implements gsfc.nssdc.cdf.CDFConstants {
 	
 	return components;
     }
-    
+
+    /**
+     * 
+     * Breaks an array of EPOCH values down into its component parts.
+     *
+     * @param epochs the array of epoch values to break down
+     * @return a 2-dimensional array, the first element being the row, while
+     *         the second element containing the epoch parts, as follwos:
+     *  <TABLE BORDER="0">
+     *    <TR><TD ALIGN="CENTER">Index</TD><TD ALIGN="CENTER">Part</TD></TR>
+     *    <TR><TD>0</TD><TD>year</TD></TR>
+     *    <TR><TD>1</TD><TD>month</TD></TR>
+     *    <TR><TD>2</TD><TD>day</TD></TR>
+     *    <TR><TD>3</TD><TD>hour</TD></TR>
+     *    <TR><TD>4</TD><TD>minute</TD></TR>
+     *    <TR><TD>5</TD><TD>second</TD></TR>
+     *    <TR><TD>6</TD><TD>msec</TD></TR>
+     *  </TABLE>
+     *  or null if null or an empty input array is detected
+     */
+    public static long [][] breakdown (double[] epochs)
+    {
+	if (epochs == null) return null;
+	int len = epochs.length;
+	if (len <= 0) return null;
+	long[][] comps = new long[len][7];
+	for (int i = 0; i < len; ++i)
+	  comps[i] = Epoch.breakdown (epochs[i]);
+	return comps;
+    }
+
     /**
      * JulianDay.
      * The year, month, and day are assumed to have already been validated.
