@@ -1,4 +1,10 @@
-// $Id: Epoch16.java,v 1.1 2012/05/18 17:47:39 liu Exp $
+// $Id: Epoch16.java,v 1.2 2014/05/14 17:38:34 liu Exp $
+/*
+ * Copyright 1996-2014 United States Government as represented by the
+ * Administrator of the National Aeronautics and Space Administration.
+ * All Rights Reserved.
+ */
+
 package gsfc.nssdc.cdf.util;
 
 import gsfc.nssdc.cdf.CDFException;
@@ -9,6 +15,11 @@ import java.util.*;
 import java.text.*;
 
 /**
+ *
+ * This class contains the handy utility routines (methods) called by the
+ * CDF applications to handle epoch data of CDF's CDF_EPOCH16 data type.
+ *
+ * @version 1.0
  *
  * <B>Example:</B>
  * <PRE> 
@@ -104,7 +115,7 @@ public class Epoch16 implements gsfc.nssdc.cdf.CDFConstants {
 	    monthStr = st.nextToken();
 	    year = Long.parseLong(st.nextToken());
 	    for (int monthX = 1; monthX <= 12; monthX++) {
-		if (monthStr.equals(_monthToken[monthX-1])) {
+		if (monthStr.equalsIgnoreCase(_monthToken[monthX-1])) {
 		    month = monthX;
 		    break;
 		}
@@ -158,15 +169,15 @@ public class Epoch16 implements gsfc.nssdc.cdf.CDFConstants {
 	throws CDFException
     {
 	StringTokenizer st = new StringTokenizer(inString, ".");
-	String date = st.nextToken();
-	String time = st.nextToken();
-	long year = 0, month = 0, day = 0, hour = 0, minute = 0, 
-	     second = 0, msec = 0, usec = 0, nsec = 0, psec = 0,
-	     fractionL = 0;
-	double fraction = 0.0, mmm; 
-	double[] epoch16 = new double[2];
-	int len;
-	try {
+        try {
+	  String date = st.nextToken();
+	  String time = st.nextToken();
+	  long year = 0, month = 0, day = 0, hour = 0, minute = 0, 
+	       second = 0, msec = 0, usec = 0, nsec = 0, psec = 0,
+	       fractionL = 0;
+	  double fraction = 0.0, mmm; 
+	  double[] epoch16 = new double[2];
+	  int len;
 	    year   = Long.parseLong(date.substring(0,4));
 	    month  = Long.parseLong(date.substring(4,6));
 	    day    = Long.parseLong(date.substring(6));
@@ -192,7 +203,7 @@ public class Epoch16 implements gsfc.nssdc.cdf.CDFConstants {
 	    if (mmm == ILLEGAL_EPOCH_VALUE)
 	      throw new CDFException(ILLEGAL_EPOCH_FIELD);
 	    return epoch16;
-	} catch (java.lang.NumberFormatException e) {
+	} catch (Exception e) {
 	    throw new CDFException(ILLEGAL_EPOCH_FIELD);
 	}
     }
@@ -233,7 +244,7 @@ public class Epoch16 implements gsfc.nssdc.cdf.CDFConstants {
 	    if (mmm == ILLEGAL_EPOCH_VALUE)
 	      throw new CDFException(ILLEGAL_EPOCH_FIELD);
 	    return epoch16;
-	} catch (java.lang.NumberFormatException e) {
+	} catch (Exception e) {
 	    throw new CDFException(ILLEGAL_EPOCH_FIELD);
 	}
     }
